#include <stdio.h>
#include <qimage.h>
#include <qmime.h>
#include <qfile.h>
#include <qsound.h>
#include <stdlib.h>
#include <eb/binary.h>
#include "qbookentry.h"


extern "C"
{
#include <libkakasi.h>
}

static const char* INSERT_MARK = "\a";
QTextCodec* g_codec;
unsigned int last_wide_font = 0;
unsigned int last_narrow_font = 0;

struct qpath_struct g_book_path [] =
  {
    {"/home/zaurus/dict/", "MAIN"},
    {"/mnt/card/dict/", "SD CARD"},
    {"/mnt/cf/dict/", "CF CARD"},
    {"/mnt/net/dict/", "NETWORK"},
    {NULL, "UNKNOWN"},
  };

struct qpath_struct g_appendix_path [] =
  {
    {"/home/zaurus/appendix/", "MAIN"},
    {"/mnt/card/appendix/", "SD CARD"},
    {"/mnt/cf/appendix/", "CF CARD"},
    {"/mnt/net/appendix/", "NETWORK"},
    {NULL, "UNKNOWN"},
  };

int
getBinary(EB_Book *book, char** buf)
{
  int length_read, size;
  size = 0;
  *buf = (char*)malloc(MAX_LENGTH);
  do
    {
      if (eb_read_binary(book, MAX_LENGTH, *buf + size, &length_read)
	  != EB_SUCCESS)
	break;
      size += length_read;
      *buf = (char*)realloc(*buf, size + MAX_LENGTH);
      if (size >= MAX_BINARY)
	break;
    }
  while (length_read > 0);
  return size;
}

int
getJpegBinary(EB_Book *book, char** buf)
{
  int length_read, size, length_next = 0;
  size = 0;
  *buf = (char*)malloc(2);
  eb_read_binary(book, 2, *buf, &length_read);
  size += 2;
  while(1) {
    *buf = (char*)realloc(*buf, size + 2);
    eb_read_binary(book, 2, *buf + size, &length_read); 
    size += 2;
    if ((*buf)[size-1] == (char)0xd9)
      break;

    *buf = (char*)realloc(*buf, size + 2);
    eb_read_binary(book, 2, *buf + size, &length_read); 
    size += 2;

    length_next = (*buf)[size-2]*0x100 + (*buf)[size-1] - 2;
    if (length_next < 0)
      return size;
    *buf = (char*)realloc(*buf, size + length_next);
    eb_read_binary(book, length_next, *buf + size, &length_read); 
    size += length_next;
  }
  return size;
}

// Hooks for EB.

EB_Error_Code
text_hook_null (EB_Book*, EB_Appendix*, void *, EB_Hook_Code, 
		int, const unsigned int*)
{
  return 0;
}

EB_Error_Code
text_hook_space (EB_Book* book, EB_Appendix*, void *, EB_Hook_Code, 
		 int, const unsigned int*)
{
  eb_write_text_string(book, " ");
  return 0;
}

EB_Error_Code
text_hook_wide_space (EB_Book* book, EB_Appendix*, void *, EB_Hook_Code, 
		      int, const unsigned int*)
{
  eb_write_text_string(book, "  ");
  return 0;
}

EB_Error_Code
text_hook_newline (EB_Book* book, EB_Appendix*,
		   void*, EB_Hook_Code, 
		   int, const unsigned int*)
{
  eb_write_text_string(book, "<p>");
  return 0;
}

EB_Error_Code
text_hook_narrow_font (EB_Book *book, EB_Appendix*,
		       void *obj, EB_Hook_Code, 
		       int, const unsigned int* argv)
{
  char c [100];
  char bitmap[EB_SIZE_NARROW_FONT_48];

  QBookEntry *be = (QBookEntry *) obj;

  EB_Error_Code rc = eb_narrow_font_character_bitmap(book, argv[0], bitmap);
  if (rc != EB_SUCCESS)
    {
      eb_write_text_string (book, " ");
      return 0;
    }

  if (last_narrow_font != argv[0])
    {
      // FIXME: there should be a better way -- from qtjiten
      QImage qim(8, 16, 1, 2, QImage::BigEndian);
      qim.setColor(0, 0xffffff);
      qim.setColor(1, 0x000000);
      for(int y=0; y<16; y++) 
	for(int x=0; x<8; x++)
	  qim.setPixel(x, y, (bitmap[y] & (1 << (7-x))) ? 1:0);
      sprintf(c, "n-font_n%x", argv[0]);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));

      qim = QImage (8, 16, 32, 2, QImage::BigEndian);
      int back = be->cg.highlight ().rgb ();
      int fore = be->cg.highlightedText ().rgb ();
      for(int y=0; y<16; y++) 
	for(int x=0; x<8; x++)
	  qim.setPixel(x, y, (bitmap[y] & (1 << (7-x))) ? fore:back);
      sprintf(c, "h-font_n%x", argv[0]);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim);
    }
  last_narrow_font = argv[0];

  sprintf(c, "<img src=\"n-font_n%x\" alt=\"GAIJI(%x)\" width=%d height=%d>",
	  argv[0], argv[0], be->getFontSize () / 2, be->getFontSize ());
  eb_write_text_string(book, c);

  return 0;
}

EB_Error_Code
text_hook_wide_font (EB_Book* book, EB_Appendix*, void *obj, 
		     EB_Hook_Code,
		     int, const unsigned* argv)
{
  char c [100];
  char bitmap[EB_SIZE_WIDE_FONT_48];
#if 0
  char image_data[EB_SIZE_FONT_IMAGE];
  size_t image_length;
#endif
  QBookEntry *be = (QBookEntry *) obj;

  EB_Error_Code rc = eb_wide_font_character_bitmap(book, argv[0], bitmap);
  if (rc != EB_SUCCESS)
    {
      eb_write_text_string (book, "  ");
      return 0;
    }

  if (last_wide_font != argv[0])
    {
      // FIXME: there should be a better way -- from qtjiten
      QImage qim(16, 16, 1, 2, QImage::BigEndian);
      qim.setColor(0, 0xffffff);
      qim.setColor(1, 0x000000);
      for(int y=0; y<16; y++) 
	for(int x=0; x<16; x++)
	  qim.setPixel(x, y, (bitmap[y*2+((x<8)?0:1)] & (1 << (7-(x%8)))) ? 1:0);
      sprintf(c, "n-font_w%x", argv[0]);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));

      qim = QImage (16, 16, 32, 2, QImage::BigEndian);
      int back = be->cg.highlight ().rgb ();
      int fore = be->cg.highlightedText ().rgb ();
      for(int y=0; y<16; y++) 
	for(int x=0; x<16; x++)
	  qim.setPixel(x, y, (bitmap[y*2+((x<8)?0:1)] & (1 << (7-(x%8))))
		       ? fore:back);
      sprintf(c, "h-font_w%x", argv[0]);
      QMimeSourceFactory::defaultFactory()->setImage(c, qim);


    }
  last_wide_font = argv[0];

  sprintf(c, "<img src=\"n-font_w%x\" alt=\"GAIJI(%x)\" width=%d height=%d>",
	  argv[0], argv[0], be->getFontSize (), be->getFontSize ());
  eb_write_text_string(book, c);

  return 0;
}

EB_Error_Code
text_hook_narrow_jisx0208 (EB_Book* book, EB_Appendix *appendix, 
				       void *container, 
				       EB_Hook_Code code, 
				       int argc, const unsigned int* argv)
{
  if (argv[0] == 0xa1e3)
    eb_write_text_string(book, "&lt;");
  else if (argv[0] == 0xa1e4)
    eb_write_text_string(book, "&gt;");
  else 
    eb_hook_euc_to_ascii(book, appendix, container, code, argc, argv);
  return 0;
}

EB_Error_Code
text_hook_begin_reference (EB_Book* book, EB_Appendix*, void*, 
			   EB_Hook_Code, 
			   int, const unsigned int*)
{
  eb_write_text_string (book, " ");
  eb_write_text_string (book, INSERT_MARK);
  eb_write_text_string (book, ">");
  return 0;
}

EB_Error_Code
text_hook_end_reference (EB_Book* book, EB_Appendix*, void *obj, 
			 EB_Hook_Code, 
			 int, const unsigned int* argv)
{
  char c[32];
  QBookEntry *be = (QBookEntry *) obj;

  sprintf (c, "%d_%d", argv[1], argv[2]);
  be->addPositionString ( new QString (c) );
  eb_write_text_string(book, "</a>");
  return 0;
}

// Image processing from qtjiten.
void
process_color (QBookEntry *be, EB_Book *book, EB_Position *p, bool inLine)
{
  if (eb_set_binary_color_graphic(book, p) != EB_SUCCESS)
    {
      qDebug("eb_set_binary_color_graphic failed");
      return;
    }
  char *buf;
  int size;
  EB_Disc_Code disc_code;

  if (be->img_type == IMG_JPEG &&
      (eb_disc_type(book, &disc_code) != EB_SUCCESS ||
       disc_code != EB_DISC_EPWING))
    size = getJpegBinary(book, &buf);
  else 
    size = getBinary(book, &buf);

  QImage qim;
  qim.loadFromData((unsigned char*)buf, size);
  free(buf);

  char c[100]; // I guess this is enough
  int w, h;
  // `scale' in qtjiten.
  w = (qim.width() * be->img_scale) / 100;
  h = (qim.height() * be->img_scale) / 100;

  // fprintf (stderr, "scale:%d,%dx%d\n", be->img_scale, w, h);
  if (inLine) 
    sprintf(c, "<img source=\"image%d\" width=%d height=%d>", 
	    be->img_no, w, h);
  else
    sprintf(c, "<p><img source=\"image%d\" width=%d height=%d><p>", 
	    be->img_no, w, h);
  eb_write_text_string(book, c);
  sprintf(c, "image%d", be->img_no);
  QMimeSourceFactory::defaultFactory()->setImage(c, qim);
  be->img_no++;
}

EB_Error_Code
hook_begin_jpeg(EB_Book*, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;

  be->img_pos.page = argv[2];
  be->img_pos.offset = argv[3];
  be->img_type = IMG_JPEG;
  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_bmp(EB_Book*, EB_Appendix*, void *obj, 
	       EB_Hook_Code, 
	       int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;

  be->img_pos.page = argv[2];
  be->img_pos.offset = argv[3];
  be->img_type = IMG_BMP;
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_color(EB_Book* book, EB_Appendix*, void *obj, 
	       EB_Hook_Code, 
	       int, const unsigned int*)
{
  QBookEntry *be = (QBookEntry *) obj;

  if (be->img_type == IMG_NA) 
    return EB_SUCCESS;
  process_color(be, book, &be->img_pos, false);
  be->img_type = IMG_NA;
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_in_color(EB_Book* book, EB_Appendix*, void *obj, 
		  EB_Hook_Code, 
		  int, const unsigned int*)
{
  QBookEntry *be = (QBookEntry *) obj;

  if (be->img_type == IMG_NA) 
    return EB_SUCCESS;
  process_color(be, book, &be->img_pos, true);
  be->img_type = IMG_NA;
  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_mono(EB_Book*, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;
  
  be->img_type = IMG_MONO;
  be->img_height = argv[2];
  be->img_width = argv[3];
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_mono(EB_Book* book, EB_Appendix*, void *obj, 
	      EB_Hook_Code, 
	      int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;  
  if (be->img_type != IMG_MONO) 
    return EB_SUCCESS;
  EB_Position p;
  p.page = argv[1];
  p.offset = argv[2];
  if (eb_set_binary_mono_graphic(book, &p, be->img_width, be->img_height) 
      != EB_SUCCESS)
    {
      qDebug("eb_set_binary_mono_graphic failed");
      return EB_SUCCESS;
    }
  char *buf;
  int size = getBinary(book, &buf);

  QImage qim;
  qim.loadFromData((unsigned char*)buf, size);
  free(buf);

  char c[100]; // I guess this is enough
  int w, h;
  // `scale' in qtjiten.
  w = (be->img_width * be->img_scale) / 100;
  h = (be->img_height * be->img_scale) / 100;  
  sprintf(c, "<img source=\"image%d\" width=%d height=%d><p>", 
	  be->img_no, w, h);
  // A bug in qte? Original w/h makes an image displayed all black.
  eb_write_text_string(book, c);
  sprintf(c, "image%d", be->img_no);
  QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth (32));
  be->img_no++;
  be->img_type = IMG_NA;
  return EB_SUCCESS;
}

EB_Error_Code
hook_begin_wave(EB_Book* book, EB_Appendix*, void *obj, 
		EB_Hook_Code, 
		int, const unsigned int* argv)
{
  QBookEntry *be = (QBookEntry *) obj;  
  
  eb_write_text_string(book, "<font color=#ff0000>");
  char c[100]; // should be enough
  int index = be->waveBegin.size();
  sprintf(c, "<a href=\"qbooksnd:%d\">", index);
  eb_write_text_string(book, c);
  be->waveBegin.resize(index+1);
  be->waveBegin[index].page = argv[2];
  be->waveBegin[index].offset = argv[3];
  be->waveEnd.resize(index+1);
  be->waveEnd[index].page = argv[4];
  be->waveEnd[index].offset = argv[5];
  return EB_SUCCESS;
}

EB_Error_Code
hook_end_wave(EB_Book* book, EB_Appendix*, void*, 
	      EB_Hook_Code, 
	      int, const unsigned int*) {
  eb_write_text_string(book, "</a></font>");

  return EB_SUCCESS;
}

//EB_Error_Code
//hook_end_multi_candidate (EB_Book* book, EB_Appendix*, void *obj, 
//			  EB_Hook_Code, 
//			  int, const unsigned int* argv)
//{
  //QBookEntry *be = (QBookEntry *) obj;
  //g_codec->toUnicode (eb_current_candidate (book));
//}

void
QBookEntry::openAppendix (QString appendix, EB_Subbook_Code sub_code)
{
  if (!appendix) return;
  uint end = appendix.findRev (')');
  if (end > 0 && end == appendix.length () - 1)
    {
      int start = appendix.findRev ('(');
      if (start > 0)
	{
	  QString memory = appendix.left (end).right (end - start - 1);
	  QString name = (const char *) appendix.left (start);

	  const char *curd;
	  int i = 0;
	  for (curd = g_appendix_path[i].path; curd != NULL;
	       curd = g_appendix_path[++i].path)
	    {
	      if (strcmp (g_appendix_path[i].name, (const char *) memory) == 0)
		{
		  if (eb_bind_appendix (&(this->appendix),
					(const char *)
					(g_appendix_path[i].path + name)) !=
		      EB_SUCCESS)
		    {
		      return;
		    }
		  eb_set_appendix_subbook (&(this->appendix), sub_code);
		}
	    }
	}
    }
}


QBookEntry::QBookEntry (QString path, QString book_name,
			int sub_book_pos,
			QString ext,
			QString app,
			QColorGroup g)
{
  EB_Subbook_Code sub_codes[EB_MAX_SUBBOOKS];
  int sub_count;

  EB_Hook heading_hooks [] = {
    {EB_HOOK_NARROW_FONT, text_hook_narrow_font}, 
    {EB_HOOK_NEWLINE, text_hook_null},
    {EB_HOOK_NARROW_JISX0208, text_hook_narrow_jisx0208},
    {EB_HOOK_WIDE_FONT, text_hook_wide_font},
    {EB_HOOK_NULL, NULL}
  };

  EB_Hook text_hooks [] = {
    {EB_HOOK_NARROW_FONT, text_hook_narrow_font}, 
    {EB_HOOK_NEWLINE, text_hook_newline},
    {EB_HOOK_WIDE_FONT, text_hook_wide_font}, 
    {EB_HOOK_NARROW_JISX0208, text_hook_narrow_jisx0208}, 
    {EB_HOOK_BEGIN_REFERENCE, text_hook_begin_reference}, 
    {EB_HOOK_END_REFERENCE, text_hook_end_reference}, 
    {EB_HOOK_BEGIN_CANDIDATE, text_hook_begin_reference},
    {EB_HOOK_END_CANDIDATE_LEAF, text_hook_end_reference},
    {EB_HOOK_END_CANDIDATE_GROUP, text_hook_end_reference},

    {EB_HOOK_BEGIN_COLOR_JPEG, hook_begin_jpeg},
    {EB_HOOK_BEGIN_COLOR_BMP, hook_begin_bmp},
    {EB_HOOK_BEGIN_IN_COLOR_JPEG, hook_begin_jpeg},
    {EB_HOOK_BEGIN_IN_COLOR_BMP, hook_begin_bmp},
    {EB_HOOK_END_COLOR_GRAPHIC, hook_end_color},
    {EB_HOOK_END_IN_COLOR_GRAPHIC, hook_end_in_color},

    {EB_HOOK_BEGIN_MONO_GRAPHIC, hook_begin_mono}, 
    {EB_HOOK_END_MONO_GRAPHIC, hook_end_mono},

    {EB_HOOK_BEGIN_WAVE, hook_begin_wave}, 
    {EB_HOOK_END_WAVE, hook_end_wave},

    {EB_HOOK_NULL, NULL}
  };

  opened = false;
  // This may be changed for korea or chinese.
  if (g_codec == NULL)
    {
      g_codec = QTextCodec::codecForName("eucJP");
    }
  font_size = 16; // default;
  img_scale = 100; // default;
  eb_initialize_hookset(&heading_hookset);
  eb_set_hooks(&heading_hookset, heading_hooks);
  eb_initialize_hookset(&text_hookset);
  eb_set_hooks(&text_hookset, text_hooks);  
  eb_initialize_book(&book);
  eb_initialize_appendix(&appendix);

  extension = ext;
  cg = g;
  pos_list.setAutoDelete (TRUE);

  if (extension == "Hiragana")
    {
      char* kakasi_args [] = {NULL, "-JH"};
      kakasi_getopt_argv (2, kakasi_args);
    }
  // fprintf (stderr, "binding: %s\n", (const char *) (path + book_name));
  if (eb_bind (&book, (const char *) path + book_name) < 0)
    {
      eb_finalize_book (&book);
      return;
    }
  eb_subbook_list (&book, sub_codes, &sub_count);
  if (sub_book_pos >= sub_count)
    {
      eb_finalize_book (&book);
      return;
    }
  eb_subbook_title2 (&book, sub_codes[sub_book_pos], title);

  this->absolute_path = path + book_name;
  this->sub_book_pos = sub_book_pos;
 
  // fprintf (stderr, "pos: %d\n", sub_book_pos);
  eb_set_subbook (&book, sub_codes[sub_book_pos]);
  openAppendix (app, sub_codes[sub_book_pos]);

  eb_set_font (&book, EB_FONT_16);

  opened = true;
  //makeMultiSearch ();
}

bool
QBookEntry::isOpened ()
{
  return opened;
}

void
QBookEntry::findMultiSearch ()
{
  EB_Multi_Search_Code multi_list[EB_MAX_MULTI_SEARCHES];
  int multi_count;
  int entry_count;
  char multi_title [EB_MAX_MULTI_TITLE_LENGTH+1];
  
  if (eb_multi_search_list (&book, multi_list, &multi_count) != EB_SUCCESS)
    {
      return;
    }
  //  qDebug ("Multi search for %s", title);
  if (multi_count == 0)
    {
      return;
    }
  for (int i = 0; i < multi_count; i++)
    {
      if (eb_multi_entry_count (&book, multi_list[i], &entry_count)
	  == EB_SUCCESS)
	{
	  multi_title[0] = '\0';
	  eb_multi_title (&book, multi_list[i], multi_title);
	  emit foundMultiSearch (this, multi_list[i],
				 QString (g_codec->toUnicode(multi_title)),
				 entry_count);
	}
    }
  return;
}

void
QBookEntry::findMultiSearchEntry (int multi_id)
{
  int entry_count;
  char entry_label [EB_MAX_MULTI_LABEL_LENGTH + 1];
  if (eb_multi_entry_count (&book, multi_id, &entry_count)
      == EB_SUCCESS)
    {
      for (int i = 0; i < entry_count; i++)
	{
	  if (eb_multi_entry_label (&book, multi_id, i, entry_label)
	      == EB_SUCCESS)
	    {
	      emit foundMultiSearchEntry (this, i,
					  QString(g_codec->
						  toUnicode(entry_label)),
					  eb_multi_entry_have_candidates
					  (&book, multi_id, i));
	    }
	}
    }
}

int
QBookEntry::multiSearch (EB_Multi_Search_Code code, QList<QString> keywords)
{
  QCString cword;
  char **multi = NULL;
  int i;

  multi = (char **) malloc (sizeof (const char *) * (keywords.count () + 1));
  for (i = 0; i < (int) keywords.count (); i++)
    {
      // cword = g_codec->fromUnicode(*keywords.at(i));
      // multi[i] = strdup ((const char *) cword);
      multi[i] = strdup ((const char *) *keywords.at (i));
    }
  multi[i] = NULL;
  eb_search_multi (&book, code, (const char **) multi);
  for (i = 0; i < (int) keywords.count (); i++)
    {
      free ((void *) multi[i]);
    }
  free (multi);
  eb_hit_list (&book, MAX_HITS_BASE, hits, &hit_count);
  return hit_count;
}

int
QBookEntry::search (QString keyword, int match_type)
{
  const char *word = NULL;
  char *word2 = NULL;
  // for keyword match.
  const char **keywords = NULL;
  QCString cword;
  int result_size = 0;

  if (match_type != KEYWORD_MATCH)
    {
      cword = g_codec->fromUnicode(keyword);
      word = cword;
    }
  switch (match_type)
    {
    case FORWARD_MATCH:
      if (keyword.length() >= MAX_HITS_THRESHOLD)
	result_size = MAX_HITS_BASE * MAX_HITS_SIZE;
      else
	result_size = MAX_HITS_BASE;
      eb_search_word (&book, word);
      break;
    case BACKWARD_MATCH:
      if (keyword.length() >= MAX_HITS_THRESHOLD)
	result_size = MAX_HITS_BASE * MAX_HITS_SIZE;
      else
	result_size = MAX_HITS_BASE;
      eb_search_endword (&book, word);
      break;
    case EXACT_MATCH:
      if (keyword.length() >= MAX_HITS_THRESHOLD)
	result_size = MAX_HITS_BASE * MAX_HITS_SIZE;
      else
	result_size = MAX_HITS_BASE;
      eb_search_exactword (&book, word);
      break;
    case KEYWORD_MATCH:
      QStringList strlist = QStringList::split (" ", keyword);
      int count = strlist.count ();
      keywords = (const char **) malloc (sizeof (const char *) * (count + 1));
      int i;
      for (i = 0; i < count; i++)
	{
	  cword = g_codec->fromUnicode(strlist[i]);
	  keywords[i] = strdup ((const char *) cword);
	}
      keywords[i] = NULL;
      if (count >= MAX_HITS_THRESHOLD)
	result_size = MAX_HITS_BASE * MAX_HITS_SIZE;
      else
	result_size = MAX_HITS_BASE;
      eb_search_keyword (&book, keywords);
      for (i = 0; i < count - 1; i++)
	{
	  free ((void *) keywords[i]);
	}
      free (keywords);
      break;
    }

  eb_hit_list (&book, result_size, hits, &hit_count);

  if (hit_count == 0 && extension == "Hiragana")
    {
      switch (match_type)
	{
	case FORWARD_MATCH:
	  word2 = kakasi_do ((char *)word);
	  eb_search_word (&book, word2);
	  kakasi_free (word2);
	  break;
	case BACKWARD_MATCH:
	  word2 = kakasi_do ((char *)word);
	  eb_search_endword (&book, word2);
	  kakasi_free (word2);
	  break;
	case EXACT_MATCH:
	  word2 = kakasi_do ((char *)word);
	  eb_search_exactword (&book, word2);
	  kakasi_free (word2);
	  break;
	case KEYWORD_MATCH:
	  QStringList strlist = QStringList::split (" ", keyword);
	  int count = strlist.count ();
	  keywords = (const char **) malloc
	    (sizeof (const char *) * (count + 1));
	  int i;
	  for (i = 0; i < count; i++)
	    {
	      cword = g_codec->fromUnicode(strlist[i]);
	      keywords[i] = (const char *)
		kakasi_do ((char *)(const char *)cword);
	    }
	  keywords[i] = NULL;
	  eb_search_keyword (&book, keywords);
	  for (i = 0; i < count; i++)
	    {
	      kakasi_free ((char *) keywords[i]);
	    }
	  free (keywords);
	  break;
	}
      eb_hit_list (&book, result_size, hits, &hit_count);
    }
  return hit_count;
}

void
QBookEntry::addPositionString (QString *str)
{
  pos_list.append (str);
}

#define QBOOK_TEMP_WAV "/home/tmp/qbook_temp.wav"

void
QBookEntry::playSound (int index)
{
  if (eb_set_binary_wave(&book, &waveBegin[index],
			 &waveEnd[index]) != EB_SUCCESS)
    {
      qDebug("eb_set_binary_wave failed");
      return;
    }
  char *buf;
  int size = getBinary(&book, &buf);
  QFile f(QBOOK_TEMP_WAV);
  f.open(IO_WriteOnly);
  QDataStream ds(&f);
  ds.writeRawBytes(buf, size);
  free(buf);
  QSound::play(QBOOK_TEMP_WAV);
  // f.remove (); QSound::play returns immediately.
}

QString
QBookEntry::getTitle ()
{
  return QString (g_codec->toUnicode(title));
}

bool
QBookEntry::isRedundantEntry (int hit_pos)
{
  int page = hits[hit_pos].text.page;
  int offset = hits[hit_pos].text.offset;
  // If same page exists earlier, it is redundant entry.
  for (int i = hit_pos - 1; i >= 0; i--)
    {
      if (hits[i].text.page == page &&
	  hits[i].text.offset == offset)
	return true;
    }
  return false;
}
  
QString
QBookEntry::getHeading (int hit_pos)
{
  QCString s;
  char text[MAX_HEADING+1];
  int text_length;
  EB_Position p0;
  
  if (eb_tell_text (&book, &p0) != EB_SUCCESS)
    p0.page = -1;
  if (eb_seek_text (&book, &hits[hit_pos].heading) != EB_SUCCESS)
    return QString (); // isNull.
  int bytes = 0;

  while (1)
    {
      if (eb_read_heading(&book, NULL, &heading_hookset, this,
			  MAX_HEADING, text, &text_length) != EB_SUCCESS)
	{
	  bytes = 0;
	  break;
	}
      bytes += text_length;
      if (text_length < 1)
	break;
      s.append(text);
    }
  if (bytes == 0)
    {
      if (p0.page >= 0)
	eb_seek_text(&book, &p0);
      return QString ();
    }
  // Remove redundant entries.
  if (isRedundantEntry (hit_pos))
    {
      return QString (); // return Null String.
    }
  else
    {
      return g_codec->toUnicode(s);
    }
}

QString
QBookEntry::getIdentity ()
{
  char s[3];
  sprintf (s, "%d", sub_book_pos);
  return absolute_path + ":" + s;
}

bool
QBookEntry::equals (QString identity)
{
  char s[3];
  sprintf (s, "%d", sub_book_pos);
  return absolute_path + ":" + s == identity;
}


EB_Position
QBookEntry::getHitPosition (int hit_pos)
{
  return hits[hit_pos].text;
}

EB_Position
QBookEntry::getLinkPosition (int link_pos)
{
  EB_Position p;
  
  if ((unsigned int)link_pos < pos_list.count ())
    {
      int page, offset;
      QString *pos = pos_list.at(link_pos);
      sscanf(pos->latin1(), "%d_%d", &page, &offset);
      p.page = page;
      p.offset = offset;
      return p;
    }
  p.page = -1;
  p.offset = -1;
  return p;
}

QString
QBookEntry::getText (EB_Position pos)
{
  QCString s, c;
  char text[MAX_TEXT+1];
  int text_length;
  EB_Position p0;

  pos_list.clear ();
  img_type = IMG_NA;
  img_no = 0;
  if (waveBegin.size () > 0)
    {
      waveBegin.truncate (0);
      waveEnd.truncate (0);
    }
  if (eb_tell_text (&book, &p0) != EB_SUCCESS)
    p0.page = -1;
  if (eb_seek_text (&book, &pos) != EB_SUCCESS)
    return QString ();
  int bytes = 0;
  EB_Appendix *app = NULL;
  if (eb_is_appendix_bound (&appendix)) app = &appendix;
  while (1)
    {
      if (eb_read_text(&book, app, &text_hookset, this,
		       MAX_TEXT, text, &text_length) != EB_SUCCESS)
	{
	  bytes = 0;
	  break;
	}
      bytes += text_length;
      if (text_length < 1)
	break;
      s.append(text);
    }
  if (bytes == 0)
    {
      if (p0.page >= 0)
	eb_seek_text(&book, &p0);
      return QString ();
    }

  int point = 0;
  int position = 0;
  QString *ref;
  char link [16];
  char height[5];
  sprintf (height, "%d", getFontSize ());

  while ((point = s.find (INSERT_MARK, 0, FALSE)) > 0)
    {
      ref = pos_list.at (position++);
      if (position <= 9)
	{
	  sprintf (link, "link%d_xpm", position);
	  s.replace ((unsigned int) point, 1,
		     QString ("<img source=\"") + link + "\" height=" + height
		     +">"+
		     + "<a href=qbook:"
		     + (const char *)*ref);
	}
      else if (position == 10)
	{
	  s.replace ((unsigned int) point, 1,
		     QString ("<img source=\"link0_xpm\" height=") + height +
		     "><a href=qbook:"
		     + (const char *)*ref);
	}
      else
	{
	  s.replace ((unsigned int) point, 1,
		     QString ("<a href=qbook:") + (const char *)*ref);	  
	}
    }
  return g_codec->toUnicode(s);
}

QBookEntry::~QBookEntry ()
{
  eb_finalize_hookset (&heading_hookset);
  eb_finalize_hookset (&text_hookset);
  eb_finalize_book (&book);
  eb_finalize_appendix (&appendix);
}
