#ifndef QBOOKENTRY_H
#define QBOOKENTRY_H

#include <qobject.h>
#include <qlist.h>
#include <qarray.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qwidget.h>
#include <eb/eb.h>
#include <eb/text.h>
#include <eb/font.h>
#include <eb/error.h>
#include <eb/appendix.h>

#define MAX_HITS_BASE	   100
#define MAX_HITS_SIZE	   3
#define MAX_HITS_THRESHOLD 3
#define MAX_HEADING	   100
#define MAX_TEXT	   1000

#define MAX_LENGTH 10000
#define MAX_BINARY 500000

#define FORWARD_MATCH  0
#define BACKWARD_MATCH 1
#define EXACT_MATCH    2
#define KEYWORD_MATCH  3

extern QTextCodec* g_codec;

enum {IMG_NA, IMG_JPEG, IMG_BMP, IMG_MONO};

class QBookEntry : public QObject
{
  Q_OBJECT
 public:
  QBookEntry (QString path,
	      QString book_name,
	      int sub_book_pos,
	      QString ext,
	      QString appendix,
	      QColorGroup g);
  ~QBookEntry ();
  // Return hit number.
  int search (QString keyword, int match_type = EXACT_MATCH);
  int multiSearch (EB_Multi_Search_Code code,
		   QList<QString> keywords);
  void setFontSize (int size) { font_size = size; }
  int getFontSize () { return font_size; }
  void setImageScale (int scale) { img_scale = scale; }
  int getImageScale () { return img_scale; }
  void findMultiSearch ();
  void findMultiSearchEntry (int multi_id);

  QString getHeading (int hit_pos);

  // To compare identity;
  QString getIdentity ();
  bool equals (QString identity);

  EB_Position getHitPosition (int hit_pos);
  EB_Position getLinkPosition (int link_pos);
  
  QString getText (EB_Position pos);
  void addPositionString (QString *str);
  EB_Book *getBook () {return &book;}
  QString getTitle ();

  QColorGroup cg;
  // image;
  EB_Position img_pos;
  int img_type;
  int img_width;
  int img_height;
  int img_no;
  int img_scale;

  // wave;
  QArray<EB_Position> waveBegin, waveEnd;
  void playSound (int index);
  bool isOpened ();
 private:
  int font_size;
  EB_Book book;
  EB_Subbook_Code sub_code;
  EB_Hookset heading_hookset;
  EB_Hookset text_hookset;
  EB_Appendix appendix;
  QList<QString> pos_list;
  
  // To identify subbook;
  QString absolute_path;
  int sub_book_pos;

  QString last_heading; // For duplication.
  
  EB_Hit hits [MAX_HITS_BASE*MAX_HITS_SIZE]; // The max hits.
  int hit_count;
  char title [EB_MAX_TITLE_LENGTH + 1];
  QString extension;

  bool isRedundantEntry (int);
  void openAppendix (QString,EB_Subbook_Code);

  bool opened;
signals:
  // this,multi_id,entry_number
  void foundMultiSearch (QBookEntry *,int,QString,int);
  // this,entry_index,label
  void foundMultiSearchEntry (QBookEntry *,int,QString,bool);
};

struct qpath_struct
{
  const char *path;
  const char *name;
};

extern struct qpath_struct g_book_path[];
extern struct qpath_struct g_appendix_path[];

#endif // QBOOKENTRY_H
