#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <qapplication.h>
#include <qclipboard.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qlistbox.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qarray.h>
#include <qstrlist.h>
#include <qtabwidget.h>
#include <qfont.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qimage.h>
#include <libkakasi.h>
#ifndef QTAPP
#include <qpe/config.h>
#else
#include <qsettings.h>
#endif
#include <eb/eb.h>

#include "qbookentry.h"
#include "qbookbrowser.h"
#include "qhistorylineedit.h"

#include "zpixmap.h"

#include "zten.h"
#include "ztenext.h"
#include "ztenbook.h"
#include "ztencandidate.h"
#include "locationbar_erase.xpm"
#include "qlistboxrichtext.h"

#include "link1.xpm"
#include "link2.xpm"
#include "link3.xpm"
#include "link4.xpm"
#include "link5.xpm"
#include "link6.xpm"
#include "link7.xpm"
#include "link8.xpm"
#include "link9.xpm"
#include "link0.xpm"

QBookHitItem::QBookHitItem (const QString &label, QBookEntry *be,
			    int hit_pos)
  : QListBoxRichText (label, label)
{
  this->be = be;
  this->hit_pos = hit_pos;
}

QBookHitItem::~QBookHitItem ()
{
}

void
Zten::readConfig ()
{
#ifdef QTAPP
  QString family = settings->readEntry ("/zten/main/FontFamily");
  int size = settings->readNumEntry ("/zten/main/FontSize");
  cur_font = family.isNull ()? font (): QFont ((const char *) family, size);

  int scale = settings->readNumEntry ("/zten/main/ImageScale");
  cur_scale = scale >= 0 ? scale : 100;

  int type = settings->readNumEntry ("/zten/main/SearchType");
  if (type >= 0)
    SearchTypeComboBox->setCurrentItem (type);

  multi_last_book_ident = settings->readEntry ("/zten/main/LastMultiBook");
  multi_last_id = settings->readNumEntry ("/zten/main/LastMultiId");
  menu_last_book_ident = settings->readEntry ("/zten/main/LastMenuBook");
  incremental_mode = settings->readBoolEntry ("/zten/main/IncrementalSearch");
  emit incrementalIsSet (incremental_mode);

  EB_Book book;
  eb_initialize_book (&book);
  for (int i = 0; i < settings->readNumEntry ("/zten/book/NumberOfBooks"); i++)
    {
      QString entryName;
      entryName.sprintf ("/zten/book/BookPath%i", i + 1);
      QString path = settings->readEntry (entryName);
      entryName.sprintf ("/zten/book/BookSubPos%i", i + 1);
      int sub_pos = settings->readNumEntry (entryName);
      entryName.sprintf ("/zten/book/BookSelected%i", i + 1);
      bool selected = settings->readBoolEntry (entryName);
      entryName.sprintf ("/zten/book/BookTitle%i", i + 1);
      QString title = settings->readEntry (entryName);
      entryName.sprintf ("/zten/book/BookName%i", i + 1);
      QString name = settings->readEntry (entryName);
      entryName.sprintf ("/zten/book/BookExtension%i", i + 1);
      QString ext = settings->readEntry (entryName);
      entryName.sprintf ("/zten/book/BookMemory%i", i + 1);
      QString memory = settings->readEntry (entryName);
      entryName.sprintf ("/zten/book/BookAppendix%i", i + 1);
      QString appendix = settings->readEntry (entryName);

      QBookCheckItem *item = new QBookCheckItem (DictionaryList,
						 "",
						 QCheckListItem::CheckBox);
      item->setText (1, title);
      item->setText (2, name);
      item->setText (3, ext);
      item->setText (4, appendix);
      item->setText (5, memory);
      item->setPath (path);
      item->setSubBookPos (sub_pos);
      if (selected)
	{
	  item->setOn (TRUE);
	}
      if (eb_bind (&book, (const char *)(path + name)) != EB_SUCCESS ||
	  eb_set_subbook (&book, sub_pos) != EB_SUCCESS)
	{
	  item->setEnabled (FALSE);
	  // item->setOn (FALSE); // Too much service.
	}
      // Move to the bottom;
      QListViewItem *last;
      for (last = DictionaryList->firstChild ();
	   last && last->itemBelow ();
	   last = last->itemBelow ());
      item->moveItem (last);
      // DictionaryList->insertItem (item);
    }
  eb_finalize_book (&book);
#else
  config->setGroup ("Main");
  QString family = config->readEntry ("FontFamily");
  int size = config->readNumEntry ("FontSize");
  cur_font = family.isNull ()? font (): QFont ((const char *) family, size);
  int scale = config->readNumEntry ("ImageScale");
  cur_scale = scale >= 0 ? scale : 100;

  int type = config->readNumEntry ("SearchType");
  if (type >= 0)
    SearchTypeComboBox->setCurrentItem (type);

  multi_last_book_ident = config->readEntry ("LastMultiBook");
  multi_last_id = config->readNumEntry ("LastMultiId");
  menu_last_book_ident = config->readEntry ("LastMenuBook");

  if (!config->hasKey ("IncrementalSearch"))
    {
      // Default is on;
      incremental_mode = true;
    }
  else
    {
      incremental_mode = config->readBoolEntry ("IncrementalSearch");
    }
  emit incrementalIsSet (incremental_mode);
  config->setGroup ("Books");
  EB_Book book;
  eb_initialize_book (&book);
  for (int i = 0; i < config->readNumEntry ("NumberOfBooks"); i++)
    {
      QString entryName;
      entryName.sprintf ("BookPath%i", i + 1);
      QString path = config->readEntry (entryName);
      entryName.sprintf ("BookSubPos%i", i + 1);
      int sub_pos = config->readNumEntry (entryName);
      entryName.sprintf ("BookSelected%i", i + 1);
      bool selected = config->readBoolEntry (entryName);      
      entryName.sprintf ("BookTitle%i", i + 1);
      QString title = config->readEntry (entryName);
      entryName.sprintf ("BookName%i", i + 1);
      QString name = config->readEntry (entryName);
      entryName.sprintf ("BookExtension%i", i + 1);
      QString ext = config->readEntry (entryName);
      entryName.sprintf ("BookAppendix%i", i + 1);
      QString appendix = config->readEntry (entryName);
      entryName.sprintf ("BookMemory%i", i + 1);
      QString memory = config->readEntry (entryName);
      QBookCheckItem *item = new QBookCheckItem (DictionaryList,
						 "",
						 QCheckListItem::CheckBox);
      item->setText (1, title);
      item->setText (2, name);
      item->setText (3, ext);
      item->setText (4, appendix);
      item->setText (5, memory);
      item->setPath (path);
      item->setSubBookPos (sub_pos);
      if (selected)
	{
	  item->setOn (TRUE);
	}
      // No need...2003/08/08
      //      if (eb_bind (&book, (const char *)(path + name)) != EB_SUCCESS ||
      //	  eb_set_subbook (&book, sub_pos) != EB_SUCCESS)
      //	{
	  // item->setEnabled (FALSE);
	  // item->setOn (FALSE); // Too much service.
      //	}
      // Move to the bottom;
      QListViewItem *last;
      for (last = DictionaryList->firstChild ();
	   last && last->itemBelow ();
	   last = last->itemBelow ());
      item->moveItem (last);
      // DictionaryList->insertItem (item);
    }
  eb_finalize_book (&book);
#endif
  setCurrentFont (cur_font);
  setImageScale (cur_scale);
  if (DictionaryList->firstChild ())
    {
#ifdef QTAPP
      int page = settings->readNumEntry ("/zten/main/StartUpPage");
#else
      config->setGroup ("Main");
      int page = config->readNumEntry ("StartUpPage");
#endif
      MainTab->setCurrentPage (page);
    }
}

void
Zten::writeConfig ()
{
#ifdef QTAPP
  settings->writeEntry ("/zten/main/FontFamily", cur_font.family());
  settings->writeEntry ("/zten/main/FontSize", cur_font.pixelSize());
  settings->writeEntry ("/zten/main/ImageScale", cur_scale);
  settings->writeEntry ("/zten/main/SearchType", SearchTypeComboBox->currentItem ());
  settings->writeEntry ("/zten/main/LastMultiBook", multi_last_book_ident);
  settings->writeEntry ("/zten/main/LastMultiId", multi_last_id);
  settings->writeEntry ("/zten/main/LastMenuBook", menu_last_book_ident);
  settings->writeEntry ("/zten/main/IncrementalSearch", incremental_mode);
  settings->writeEntry ("/zten/main/StartUpPage", MainTab->currentPageIndex ());
  
  int number = 0;
  QListViewItemIterator it (DictionaryList);
  QBookCheckItem *bi;
  for (; it.current(); ++it, ++number)
    {
      QString entryName;
      bi = (QBookCheckItem *) it.current ();
      entryName.sprintf ("/zten/book/BookPath%i", number + 1);
      settings->writeEntry (entryName, bi->getPath ());
      entryName.sprintf ("/zten/book/BookSubPos%i", number + 1);
      settings->writeEntry (entryName, bi->getSubBookPos ());
      entryName.sprintf ("/zten/book/BookSelected%i", number + 1);
      settings->writeEntry (entryName, bi->isOn ());      
      entryName.sprintf ("/zten/book/BookTitle%i", number + 1);
      settings->writeEntry (entryName, bi->text (1));
      entryName.sprintf ("/zten/book/BookName%i", number + 1);
      settings->writeEntry (entryName, bi->text (2));
      entryName.sprintf ("/zten/book/BookExtension%i", number + 1);
      settings->writeEntry (entryName, bi->text (3));
      entryName.sprintf ("/zten/book/BookAppendix%i", number + 1);
      settings->writeEntry (entryName, bi->text (4));      
      entryName.sprintf ("/zten/book/BookMemory%i", number + 1);
      settings->writeEntry (entryName, bi->text (5));
    }
  settings->writeEntry ("/zten/book/NumberOfBooks", number);
#else  

  config->setGroup ("Main");
  config->writeEntry ("FontFamily", cur_font.family());
  config->writeEntry ("FontSize", cur_font.pixelSize());
  config->writeEntry ("ImageScale", cur_scale);
  config->writeEntry ("SearchType", SearchTypeComboBox->currentItem ());
  config->writeEntry ("LastMultiBook", multi_last_book_ident);
  config->writeEntry ("LastMultiId", multi_last_id);
  config->writeEntry ("LastMenuBook", menu_last_book_ident);
  config->writeEntry ("IncrementalSearch", incremental_mode);
  config->writeEntry ("StartUpPage", MainTab->currentPageIndex ());
  
  config->setGroup ("Books");
  int number = 0;
  QListViewItemIterator it (DictionaryList);
  QBookCheckItem *bi;
  for (; it.current(); ++it, ++number)
    {
      QString entryName;
      bi = (QBookCheckItem *) it.current ();
      entryName.sprintf ("BookPath%i", number + 1);
      config->writeEntry (entryName, bi->getPath ());
      entryName.sprintf ("BookSubPos%i", number + 1);
      config->writeEntry (entryName, bi->getSubBookPos ());
      entryName.sprintf ("BookSelected%i", number + 1);
      config->writeEntry (entryName, bi->isOn ());      
      entryName.sprintf ("BookTitle%i", number + 1);
      config->writeEntry (entryName, bi->text (1));
      entryName.sprintf ("BookName%i", number + 1);
      config->writeEntry (entryName, bi->text (2));
      entryName.sprintf ("BookExtension%i", number + 1);
      config->writeEntry (entryName, bi->text (3));
      entryName.sprintf ("BookAppendix%i", number + 1);
      config->writeEntry (entryName, bi->text (4));
      entryName.sprintf ("BookMemory%i", number + 1);
      config->writeEntry (entryName, bi->text (5));
    }
  config->writeEntry ("NumberOfBooks", number);
#endif
}

void
Zten::entryListEscape ()
{
  toggleContentForce (FALSE);
  LineEdit->setFocus();
}

void
Zten::entryListUp ()
{
  QListBoxItem *item;
  int i = EntryList->currentItem ();
  if (i >= 0)
    {
      if ((item = EntryList->item(i)->prev()))
	{
	  EntryList->setCurrentItem (item);
	}
      else
	{
	  // Too much service?
	  // LineEdit->setFocus();
	}
    }
}

void
Zten::entryListDown ()
{
  QListBoxItem *item;
  int i = EntryList->currentItem ();
  if (i >= 0)
    {
      if ((item = EntryList->item(i)->next()))
	{	
	  EntryList->setCurrentItem (item);
	}
    }  
}


void
Zten::entryListUp2 ()
{
  QListBoxItem *item;
  int i = EntryList2->currentItem ();
  if (i >= 0)
    {
      if ((item = EntryList2->item(i)->prev()))
	{
	  EntryList2->setCurrentItem (item);
	}
      else
	{
	  // Too much service?
	  // LineEdit->setFocus();
	}
    }
}

void
Zten::entryListDown2 ()
{
  QListBoxItem *item;
  int i = EntryList2->currentItem ();
  if (i >= 0)
    {
      if ((item = EntryList2->item(i)->next()))
	{	
	  EntryList2->setCurrentItem (item);
	}
    }  
}

void
Zten::contentLineDown ()
{
  if (displaying)
    {
      ContentBrowser->scrollBy (0, cur_font.pixelSize());
    }
  else
    {
      toggleContentForce (TRUE);
    }
}

void
Zten::contentLineUp ()
{
  if (displaying)
    ContentBrowser->scrollBy (0, -cur_font.pixelSize());
  else
    {
      toggleContentForce (TRUE);
    }  
}

void
Zten::contentLineDown2 ()
{
  if (displaying2)
    {
      ContentBrowser2->scrollBy (0, cur_font.pixelSize());
    }
  else
    {
      toggleContentForce2 (TRUE);
    }
}

void
Zten::contentLineDown3 ()
{
  ContentBrowser3->scrollBy (0, cur_font.pixelSize());
}

void
Zten::contentLineUp2 ()
{
  if (displaying2)
    ContentBrowser2->scrollBy (0, -cur_font.pixelSize());
  else
    {
      toggleContentForce2 (TRUE);
    }  
}

void
Zten::contentLineUp3 ()
{
  ContentBrowser3->scrollBy (0, -cur_font.pixelSize());
}

void
Zten::contentPageUp ()
{
  if (displaying)
    {
      if (ContentBrowser->contentsY () == 0)
	{
	  entryListEscape ();
	}
      else
	{
	  ContentBrowser->scrollBy (0, ContentBrowser->height () * -8 / 10);
	}
    }
  else
    {
      entryListEscape ();
    }
}

void
Zten::contentPageDown ()
{
  if (displaying)
    {
      ContentBrowser->scrollBy (0, ContentBrowser->height () * 8 / 10);
    }
  else
    {
      toggleContentForce (TRUE);
    }
}

void
Zten::contentPageUp2 ()
{
  if (displaying2)
    {
      if (ContentBrowser2->contentsY () == 0)
	{
	  MultiTab->setCurrentPage (0);
	}
      else
	{
	  ContentBrowser2->scrollBy (0, ContentBrowser2->height () * -8 / 10);
	}
    }
  else
    {
      MultiTab->setCurrentPage (0);
    }
}

void
Zten::contentPageUp3 ()
{
  ContentBrowser3->scrollBy (0, ContentBrowser3->height () * -8 / 10);
}

void
Zten::contentPageDown3 ()
{
  ContentBrowser3->scrollBy (0, ContentBrowser3->height () * 8 / 10);
}

void
Zten::contentPageDown2 ()
{
  if (displaying2)
    {
      ContentBrowser2->scrollBy (0, ContentBrowser2->height () * 8 / 10);
    }
  else
    {
      toggleContentForce2 (TRUE);
    }
}

void
Zten::toggleDirection ()
{
  if (BookContentLayout->direction () == QBoxLayout::LeftToRight)
    {
      EntryList->setMinimumSize(QSize(ContentBrowser->width (),
				      ContentBrowser->height () / 3));
      BookContentLayout->setDirection (QBoxLayout::TopToBottom);
    }
  else
    {
      EntryList->setMinimumSize(QSize(ContentBrowser->width () / 3,
				      ContentBrowser->height ()));      
      BookContentLayout->setDirection (QBoxLayout::LeftToRight);
    }
}

void
Zten::toggleDirection2 ()
{
  if (BookContentLayout2->direction () == QBoxLayout::LeftToRight)
    {
      EntryList2->setMinimumSize(QSize(ContentBrowser2->width (),
				       ContentBrowser2->height () / 3));
      BookContentLayout2->setDirection (QBoxLayout::TopToBottom);
    }
  else
    {
      EntryList2->setMinimumSize(QSize(ContentBrowser2->width () / 3,
				       ContentBrowser2->height ()));      
      BookContentLayout2->setDirection (QBoxLayout::LeftToRight);
    }
}

void
Zten::resizeEvent (QResizeEvent* e)
{
  QSize s = e->size ();
  if (s.width () > s.height ())
    {
      BookContentLayout->setDirection (QBoxLayout::LeftToRight);
      BookContentLayout2->setDirection (QBoxLayout::LeftToRight);
    }
  else
    {
      BookContentLayout->setDirection (QBoxLayout::TopToBottom);      
      BookContentLayout2->setDirection (QBoxLayout::TopToBottom);
    }
  // FIXME: hard coding.
  EntryList->setMinimumSize(QSize(s.width () / 3,
				  s.height () / 4));
  EntryList2->setMinimumSize(QSize(s.width () / 3,
				   s.height () / 4));
  ZtenBase::resizeEvent (e);
}

void
Zten::toggleContent ()
{
  if (displaying)
    {
      if (ContentBrowser->hasFocus ()) EntryList->setFocus();
      ContentBrowser->hide ();
      displaying = FALSE;
    }
  else
    {
      displaying = TRUE;
      int i = EntryList->currentItem ();
      if (i >= 0 && redisplay (EntryList->item(i)))
	{
	  ContentBrowser->show ();
	}
    }
}

void
Zten::toggleContentForce (bool force)
{
  if (force != displaying) toggleContent ();
}

void
Zten::toggleContent2 ()
{
  if (displaying2)
    {
      if (ContentBrowser2->hasFocus ()) EntryList2->setFocus();
      ContentBrowser2->hide ();
      displaying2 = FALSE;
    }
  else
    {
      displaying2 = TRUE;
      int i = EntryList2->currentItem ();
      if (i >= 0 && redisplay (EntryList2->item(i)))
	{
	  ContentBrowser2->show ();
	}
    }
}

void
Zten::toggleContentForce2 (bool force)
{
  if (force != displaying2) toggleContent2 ();
}

void
Zten::linkJump (int pos)
{
  EB_Position p = current_book->getLinkPosition (pos);
  if (p.page >= 0) goTo (p);
}

bool
Zten::eventFilter (QObject *o, QEvent *e)
{
  if (e->type () == QEvent::KeyPress)
    {
      if (MainTab->currentPageIndex () == 1)
	{
	  QKeyEvent *k = (QKeyEvent *)e;
	  // qDebug( "Key press %d", k->key() );
	  switch (k->key ())
	    {
	    case Qt::Key_1:
	    case Qt::Key_2:
	    case Qt::Key_3:
	    case Qt::Key_4:
	    case Qt::Key_5:
	    case Qt::Key_6:
	    case Qt::Key_7:
	    case Qt::Key_8:
	    case Qt::Key_9:
	      linkJump (k->key () - Qt::Key_1);
	      break;
	    case Qt::Key_0:
	      linkJump (9);
	      break;
	    case Qt::Key_B:
	      if (k->state () & ShiftButton)
		{
		  // 'B'
		  back ();
		}
	      break;
	    case Qt::Key_F:
	      if (k->state () & ShiftButton)
		{
		  // 'F'
		  forward ();
		}
	      break;
	    case Qt::Key_J:
	    case Qt::Key_N:
	    case Qt::Key_Down:
	      entryListDown ();
	      break;
	    case Qt::Key_K:
	    case Qt::Key_P:
	    case Qt::Key_Up:
	      entryListUp ();
	      break;
	    case Qt::Key_Space:
	      contentPageDown ();
	      break;
	    case Qt::Key_Backspace:
	      contentPageUp ();
	      break;
	    case Qt::Key_Q:
	      entryListEscape ();
	      break;
	    case Qt::Key_Delete:
	      clearLine ();
	      break;
	    case Qt::Key_V:
	      toggleContent ();
	      break;
	    case Qt::Key_D:
	      toggleDirection ();
	      break;
	    case Qt::Key_Return:
	      contentLineDown ();
	      break;
	    case Qt::Key_Minus:
	      contentLineUp ();
	      break;
	    case Qt::Key_Slash:
	      down ();
	      break;
	    default:
	      return FALSE;
	    }
	  return TRUE;
	}
      else if (MainTab->currentPageIndex () == 2)
	{
	  // MultiSearch
	  QKeyEvent *k = (QKeyEvent *)e;
	  // qDebug( "Key press %d", k->key() );
	  switch (k->key ())
	    {
	    case Qt::Key_1:
	    case Qt::Key_2:
	    case Qt::Key_3:
	    case Qt::Key_4:
	    case Qt::Key_5:
	    case Qt::Key_6:
	    case Qt::Key_7:
	    case Qt::Key_8:
	    case Qt::Key_9:
	      linkJump (k->key () - Qt::Key_1);
	      break;
	    case Qt::Key_0:
	      linkJump (9);
	      break;
	    case Qt::Key_B:
	      if (k->state () & ShiftButton)
		{
		  // 'B'
		  back ();
		}
	      break;
	    case Qt::Key_F:
	      if (k->state () & ShiftButton)
		{
		  // 'F'
		  forward ();
		}
	      break;
	    case Qt::Key_J:
	    case Qt::Key_N:
	    case Qt::Key_Down:
	      entryListDown2 ();
	      break;
	    case Qt::Key_K:
	    case Qt::Key_P:
	    case Qt::Key_Up:
	      entryListUp2 ();
	      break;
	    case Qt::Key_Space:
	      contentPageDown2 ();
	      break;
	    case Qt::Key_Backspace:
	      contentPageUp2 ();
	      break;
	    case Qt::Key_Q:
	    case Qt::Key_Delete:
	      MultiTab->setCurrentPage (0);
	      break;
	    case Qt::Key_V:
	      toggleContent2 ();
	      break;
	    case Qt::Key_D:
	      toggleDirection2 ();
	      break;
	    case Qt::Key_Return:
	      contentLineDown2 ();
	      break;
	    case Qt::Key_Minus:
	      contentLineUp2 ();
	      break;
	    default:
	      return FALSE;
	    }
	  return TRUE;	  
	}
      else if (MainTab->currentPageIndex () == 3)
	{
	  // MultiSearch
	  QKeyEvent *k = (QKeyEvent *)e;
	  // qDebug( "Key press %d", k->key() );
	  switch (k->key ())
	    {
	    case Qt::Key_1:
	    case Qt::Key_2:
	    case Qt::Key_3:
	    case Qt::Key_4:
	    case Qt::Key_5:
	    case Qt::Key_6:
	    case Qt::Key_7:
	    case Qt::Key_8:
	    case Qt::Key_9:
	      linkJump (k->key () - Qt::Key_1);
	      break;
	    case Qt::Key_0:
	      linkJump (9);
	      break;
	    case Qt::Key_B:
	      if (k->state () & ShiftButton)
		{
		  // 'B'
		  back ();
		}
	      break;
	    case Qt::Key_F:
	      if (k->state () & ShiftButton)
		{
		  // 'F'
		  forward ();
		}
	      break;
	    case Qt::Key_Space:
	      contentPageDown3 ();
	      break;
	    case Qt::Key_Backspace:
	      contentPageUp3 ();
	      break;
	    case Qt::Key_Q:
	    case Qt::Key_Delete:
	      MenuSearch->setFocus ();
	      break;
	    case Qt::Key_Return:
	      contentLineDown3 ();
	      break;
	    case Qt::Key_Minus:
	      contentLineUp3 ();
	      break;
	    default:
	      return FALSE;
	    }
	  return TRUE;	  
	}
    }
  return QWidget::eventFilter (o, e);
}

Zten::Zten(QWidget* parent, const char* name, WFlags fl)
  : ZtenBase(parent, name, fl)
{
  QMimeSourceFactory::defaultFactory ()->setImage ("link1_xpm",
						    QImage (link1_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link2_xpm",
						    QImage (link2_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link3_xpm",
						    QImage (link3_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link4_xpm",
						    QImage (link4_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link5_xpm",
						    QImage (link5_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link6_xpm",
						    QImage (link6_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link7_xpm",
						    QImage (link7_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link8_xpm",
						    QImage (link8_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link9_xpm",
						    QImage (link9_xpm));
  QMimeSourceFactory::defaultFactory ()->setImage ("link0_xpm",
						    QImage (link0_xpm));
  eb_initialize_library();
#ifdef QTAPP
  settings = new QSettings ();
#else
  config = new Config ("zten");
#endif
  displaying = TRUE;
  displaying2 = TRUE;
  
  EntryList->setHScrollBarMode( QScrollView::AlwaysOff );
  ContentBrowser->setHScrollBarMode( QTextBrowser::AlwaysOff );

  EntryList2->setHScrollBarMode( QScrollView::AlwaysOff );
  ContentBrowser2->setHScrollBarMode( QTextBrowser::AlwaysOff );

  ContentBrowser3->setHScrollBarMode( QTextBrowser::AlwaysOff );

  BookContentLayout->setStretchFactor(EntryList, 1);
  BookContentLayout->setStretchFactor(ContentBrowser, 2);

  BookContentLayout2->setStretchFactor(EntryList2, 1);
  BookContentLayout2->setStretchFactor(ContentBrowser2, 2);

  SearchTypeComboBox->insertItem (tr ("Head"));
  SearchTypeComboBox->insertItem (tr ("Tail"));
  SearchTypeComboBox->insertItem (tr ("Exact"));
  SearchTypeComboBox->insertItem (tr ("Word"));

  books.setAutoDelete (TRUE);
  ms_list.setAutoDelete (TRUE);
  // menu_books.setAutoDelete (TRUE);
  DelButton->setPixmap (ZPIXMAP(locationbar_erase_xpm));

  current_book = NULL;
  history.resize (0);
  history_current = -1;

  // last selected;
  multi_last_book_ident = QString ();
  multi_last_id = -1;
  menu_last_book_ident = QString ();
 
  connect ((QObject *)MainTab, SIGNAL (currentChanged (QWidget*)),
	   this, SLOT (tabChanged(QWidget*)));
  connect ((QObject *)LookupButton, SIGNAL (clicked()),
	   this, SLOT (activateAndSearch()));
  connect ((QObject *)DelButton, SIGNAL (clicked()),
	   this, SLOT (clearLine()));
  connect (LineEdit, SIGNAL (returnPressed ()), this, SLOT (search()));  
  connect (LineEdit, SIGNAL (textChanged (const QString &)),
	   this, SLOT (searchIncremental (const QString &)));
  connect (EntryList, SIGNAL (currentChanged (QListBoxItem *)), 
	   this, SLOT (itemSelected (QListBoxItem *)));
  connect (EntryList, SIGNAL (pressed (QListBoxItem *)), 
	   this, SLOT (itemPressed (QListBoxItem *)));

  connect (EntryList2, SIGNAL (currentChanged (QListBoxItem *)), 
	   this, SLOT (itemSelected2 (QListBoxItem *)));
  connect (EntryList2, SIGNAL (pressed (QListBoxItem *)), 
	   this, SLOT (itemPressed2 (QListBoxItem *)));
  
  connect (MultiSearch, SIGNAL (activated (int)),
	   this, SLOT (multiSelected (int)));
  connect (MultiLookup, SIGNAL (clicked ()),
	   this, SLOT (multiSearch ()));

  connect (MenuSearch, SIGNAL (activated (int)),
	   this, SLOT (menuSelected (int)));  

  connect (LineEdit1, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit2, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit3, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit4, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
  connect (LineEdit5, SIGNAL (returnPressed ()), this, SLOT (multiSearch()));  
			      
  connect (ContentBrowser, SIGNAL (goTo (EB_Position)),
	   this, SLOT (goTo (EB_Position)));
  connect (ContentBrowser, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));

  connect (ContentBrowser2, SIGNAL (goTo (EB_Position)),
	   this, SLOT (goTo (EB_Position)));
  connect (ContentBrowser2, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));

  connect (ContentBrowser3, SIGNAL (goTo (EB_Position)),
	   this, SLOT (goTo (EB_Position)));
  connect (ContentBrowser3, SIGNAL (playSound (int)),
	   this, SLOT (playSound (int)));

  connect (MultiCandidate1, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate1 ()));
  connect (MultiCandidate2, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate2 ()));
  connect (MultiCandidate3, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate3 ()));
  connect (MultiCandidate4, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate4 ()));
  connect (MultiCandidate5, SIGNAL (clicked ()),
	   this, SLOT (multiCandidate5 ()));

  candidate1 = QString ();
  candidate2 = QString ();
  candidate3 = QString ();
  candidate4 = QString ();
  candidate5 = QString ();

  //  connect (QApplication::clipboard(), SIGNAL (dataChanged ()),
  //	   this, SLOT (searchClipboard ()));

  opened = FALSE;  

  EntryList->installEventFilter (this);
  // SearchTypeComboBox->installEventFilter (this);
  
  ContentBrowser->installEventFilter (this);
  EntryList2->installEventFilter (this);
  ContentBrowser2->installEventFilter (this);
  ContentBrowser3->installEventFilter (this);
  
  toggleContentForce (FALSE);
  toggleContentForce2 (FALSE);
}

Zten::~Zten()
{
  writeConfig ();
#ifdef QTAPP
  delete settings;
#else  
  delete config;
#endif
  eb_initialize_library();
}

void
Zten::setImageScale (int scale)
{
  QBookEntry *be;
  cur_scale = scale;
  for (be = books.first (); be != NULL; be = books.next ())
    {
      be->setImageScale (scale);
    }
  // fprintf (stderr, "history:%d\n", history_current);
  if (history_current >= 0)
    goToNoHistory (history[history_current]);
}
  
void
Zten::setCurrentFont (QFont font)
{
  cur_font = font;
  DictionaryList ->setFont (font);
  EntryList->setFont (font);
  EntryList2->setFont (font);
  ContentBrowser->setFont (font);
  ContentBrowser2->setFont (font);
  ContentBrowser3->setFont (font);
  QBookEntry *be;
  
  for (be = books.first (); be != NULL; be = books.next ())
    {
      be->setFontSize (font.pixelSize ());
    }
  if (history_current >= 0)
    goToNoHistory (history[history_current]);
}

void
Zten::selectBooks ()
{
  ZtenBook b (this, 0, true);

  QListViewItemIterator it (DictionaryList);
  QBookCheckItem *bi;
  for (; it.current();++it)
    {
      bi = (QBookCheckItem *) it.current ();
      b.addSelectedBook (bi);
    }
#ifdef QTAPP  
  b.show ();
#else
  b.showMaximized ();
#endif  
  b.exec ();  
}

void
Zten::reload ()
{
  closeBooks ();
  tabChanged (NULL);
}

void
Zten::clearBooks ()
{
  DictionaryList->clear ();
}

void
Zten::addBook(QBookItem *item)
{
  QBookCheckItem *newItem = new QBookCheckItem (DictionaryList,
						"",
						QCheckListItem::CheckBox);

  newItem->setOn (item->getChecked ());
  newItem->setText (1, item->text (0));
  newItem->setText (2, item->text (1));
  newItem->setText (3, item->getExtension ());
  newItem->setSubBookPos (item->getSubBookPos ());
  newItem->setPath (item->getPath ());
  newItem->setText (4, item->getAppendix ());
  newItem->setText (5, item->text (2));

  // Move to the bottom;
  QListViewItem *last;
  for (last = DictionaryList->firstChild ();
       last && last->itemBelow ();
       last = last->itemBelow ());
  newItem->moveItem (last);
}

void
Zten::setExtension ()
{
  QListViewItem *item = DictionaryList->selectedItem ();
  if (item)
    {
      ZtenExtension ext (item->text(3), item->text(4), this, 0, true);
      ext.showMaximized ();
      ext.exec ();
    }
}

void
Zten::listUp ()
{
  QListViewItem *item = DictionaryList->selectedItem ();
  if (item && item->itemAbove ())
    {
      item->itemAbove()->moveItem (item);
    }
  DictionaryList->ensureItemVisible (DictionaryList->selectedItem ());  
}

void
Zten::listDown ()
{
  QListViewItem *item = DictionaryList->selectedItem ();
  if (item && item->itemBelow ())
    {
      item->moveItem (item->itemBelow ());
    }
  DictionaryList->ensureItemVisible (DictionaryList->selectedItem ());  
}

void
Zten::deleteBook ()
{
  QListViewItem *item = DictionaryList->selectedItem ();
  if (item) DictionaryList->removeItem (item);
}

bool
Zten::redisplay (QListBoxItem *item)
{
  if (item)
    {
      QBookHitItem *bi = (QBookHitItem *)item;
      if (bi == NULL)
	return FALSE;
      QBookEntry *be;
      be = bi->getBookEntry ();
      if (be)
	{
	  history.resize (0);
	  history_current = -1;
	  emit noHistory ();
	  current_book = be;
	  goTo(be->getHitPosition(bi->getHitPos ()));
	  return TRUE;
	}
    }
  return FALSE;
}

void
Zten::itemSelected (QListBoxItem *item)
{
  if (item)
    {
      EntryList->setSelectionMode(QListBox::Single);
      if (displaying) redisplay (item);
    }
}

void
Zten::itemPressed (QListBoxItem *item)
{
  if (item)
    {
      EntryList->setSelectionMode(QListBox::Single);
      toggleContentForce (TRUE);
    }
}

void
Zten::itemSelected2 (QListBoxItem *item)
{
  if (item)
    {
      EntryList2->setSelectionMode(QListBox::Single);
      if (displaying2) redisplay (item);
    }
}

void
Zten::itemPressed2 (QListBoxItem *item)
{
  if (item)
    {
      EntryList2->setSelectionMode(QListBox::Single);
      toggleContentForce2 (TRUE);
    }
}

void
Zten::lookup (QString key)
{
  LineEdit->setEditText (key);
  if (MainTab->currentPageIndex () != 1)
    {
      MainTab->setCurrentPage (1);
    }
  search ();
  LineEdit->setFocus ();
}

void
Zten::setIncremental (bool on)
{
  incremental_mode = on;
  emit incrementalIsSet (on);
}

void
Zten::searchIncremental (const QString & word)
{
  if (incremental_mode)
    {
      toggleContentForce (FALSE);
      EntryList->clear();
      if (word.length () >= 1)
	{
	  QBookEntry *be;
	  int matches;
	  for (be = books.first (); be != NULL; be = books.next ())
	    {
	      matches = be->search (word, SearchTypeComboBox->currentItem ());
	      for (int i = 0 ; i < matches; i ++)
		{
		  QString heading = be->getHeading (i);
		  if (!heading.isNull())
		    {
		      EntryList->insertItem (new QBookHitItem
					     (heading + " [" +
					      be->getTitle () + "]",
					      be, i), -1);
		    }
		}
	    }
	}
      else
	{
	  ContentBrowser->setText ("");
	}
    }
}

void
Zten::activateAndSearch()
{
  LineEdit->activate ();
  search ();
}

void
Zten::clearLine ()
{
  LineEdit->setFocus ();
  LineEdit->setEditText ("");
}

void
Zten::search ()
{
  QBookEntry *be;
  int total_matches = 0;
  int matches;
  QBookEntry *first_match = NULL;
  QString word = LineEdit->currentText();
  
  EntryList->clear();
  if (word.length () >= 1)
    {
      for (be = books.first (); be != NULL; be = books.next ())
	{
	  // search type equals to SearchTypeComboBox->currentItem ();
	  matches = be->search (word,
				SearchTypeComboBox->currentItem ());
	  for (int i = 0 ; i < matches; i ++)
	    {
	      if (first_match == NULL)
		{
		  first_match = be;
		}
	      QString heading = be->getHeading (i);
	      if (!heading.isNull())  
		EntryList->insertItem (new QBookHitItem
				       (heading + " [" +
					be->getTitle () + "]",
					be, i), -1);
	    }
	  total_matches += matches;
	}
    }
  if (total_matches == 0)
    {
      EntryList->insertItem (new QBookHitItem ("No matches", NULL, -1));
      EntryList->setSelectionMode (QListBox::NoSelection);
    }
  else
    {
      EntryList->setSelectionMode(QListBox::Single);
      EntryList->setSelected(0, true);
      EntryList->setFocus();
      
      toggleContentForce (TRUE);
    }
}

void
Zten::goTo (EB_Position pos)
{
  goToNoHistory (pos);

  if (history_current == (int) history.size() - 1)
    {
      history.resize(history.size() + 1);
    }
  history[++history_current] = pos;
  if (history.size () != 1) emit makeHistory ();
}

void
Zten::setCandidateLabel (int index, QString text, QString label)
{
  switch (index)
    {
    case 1:
      candidate1 = QString (text);
      LineEdit1->setText (label);
      break;
    case 2:
      candidate2 = QString (text);
      LineEdit2->setText (label);
      break;
    case 3:
      candidate3 = QString (text);
      LineEdit3->setText (label);
      break;
    case 4:
      candidate4 = QString (text);
      LineEdit4->setText (label);
      break;
    case 5:
      candidate5 = QString (text);
      LineEdit5->setText (label);
      break;
    }
}

void
Zten::multiCandidate1 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearch->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 0,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 1, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString)),
	       this, SLOT (setCandidateLabel (int, QString, QString)));
#ifdef QTAPP
      c.show ();
#else      
      c.showMaximized ();
#endif
      c.exec ();  
    }
}

void
Zten::multiCandidate2 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearch->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 1,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 2, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString)),
	       this, SLOT (setCandidateLabel (int, QString, QString)));
#ifdef QTAPP
      c.show ();
#else      
      c.showMaximized ();
#endif
      c.exec ();  
    }
}

void
Zten::multiCandidate3 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearch->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 2,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 3, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString)),
	       this, SLOT (setCandidateLabel (int, QString, QString)));
#ifdef QTAPP
      c.show ();
#else      
      c.showMaximized ();
#endif
      c.exec ();  
    }
}

void
Zten::multiCandidate4 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearch->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 3,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 4, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString)),
	       this, SLOT (setCandidateLabel (int, QString, QString)));
#ifdef QTAPP
      c.show ();
#else      
      c.showMaximized ();
#endif      
      c.exec ();  
    }
}

void
Zten::multiCandidate5 ()
{
  QBookMultiSearch *ms = ms_list.at (MultiSearch->currentItem ());
  EB_Position pos;
  EB_Book *book = ms->getBookEntry ()->getBook ();
  if (eb_multi_entry_candidates (book,
				 ms->getMultiId (),
				 4,
				 &pos) == EB_SUCCESS)
    {
      ZtenCandidate c (book, pos, 5, this, 0, true);
      connect (&c, SIGNAL (candidateDecided (int, QString, QString)),
	       this, SLOT (setCandidateLabel (int, QString, QString)));      
#ifdef QTAPP
      c.show ();
#else      
      c.showMaximized ();
#endif
      c.exec ();  
    }
}

void
Zten::playSound (int index)
{
  current_book->playSound (index);
}

void
Zten::goToNoHistory (EB_Position pos)
{
  current_browser->setText (current_book->getText (pos));
}

void
Zten::back ()
{
  emit followHistory ();
  if (history_current >= 1)
    {
      history_current--;
      EB_Position p = history[history_current];
      goToNoHistory (p);
    }
  if (history_current == 0)
    {
      emit beginningOfHistory ();
    }
  //  fprintf (stderr, "current= %d\n", history_current);
  //  fprintf (stderr, "size= %d\n", history.size());  
}

void
Zten::forward ()
{
  emit followHistory ();
  if (history_current < (int) history.size() - 1)
    {
      history_current++;
      EB_Position p = history[history_current];
      goToNoHistory (p);
    }
  if (history_current == (int) history.size() - 1)
    {
      emit endOfHistory ();
    }
  //  fprintf (stderr, "current= %d\n", history_current);
  //  fprintf (stderr, "size= %d\n", history.size());
}

void
Zten::down ()
{
  if (current_book)
    {
      EB_Position p;
      if (eb_tell_text(current_book->getBook(), &p) != EB_SUCCESS)
	return;
      int a = p.page * 2048 + p.offset ;
      p.page = a/2048;
      p.offset = a%2048;
      goTo(p);
      // EntryList->clear();
    }
}

int
Zten::onItemPos (QBookCheckItem *item)
{
  int index = 0;
  QListViewItemIterator it (DictionaryList);
  QBookCheckItem *bi;
  for (; it.current();++it)
    {
      bi = (QBookCheckItem *) it.current ();
      if (bi->isOn () && bi->isEnabled ())
	{
	  if (item == bi) return index;
	  index ++;
	}
    }
  return -1;
}

void
Zten::gotoMenu (QBookEntry *be)
{
  EB_Position p;
  if (eb_menu(be->getBook(), &p) != EB_SUCCESS)
    return;
  history.resize (0);
  history_current = -1;
  emit noHistory ();
  current_browser = ContentBrowser3;
  goTo (p);
}

void
Zten::multiSelected (int index)
{

  QBookEntry *be = NULL;
  QBookMultiSearch *ms = NULL;
  multi_number = 0;
  
  if (ms_list.count () > 0)
    {
      MultiSearch->setEnabled (TRUE);
      ms = ms_list.at (index);
      be = ms->getBookEntry ();
      MultiLookup->setEnabled (TRUE);
    }
  else
    {
      MultiSearch->setEnabled (FALSE);
    }
  // FIXME: Hard coding.
  MultiLabel1->setText ("");
  LineEdit1->setEnabled (FALSE);
  LineEdit1->setText ("");
  MultiCandidate1->setEnabled (FALSE);
  if (!candidate1.isNull ())
    candidate1 = QString ();

  MultiLabel2->setText ("");
  LineEdit2->setEnabled (FALSE);
  LineEdit2->setText ("");
  MultiCandidate2->setEnabled (FALSE);
  if (!candidate2.isNull ())
    candidate2 = QString ();
  
  MultiLabel3->setText ("");
  LineEdit3->setEnabled (FALSE);
  LineEdit3->setText ("");
  MultiCandidate3->setEnabled (FALSE);
  if (!candidate3.isNull ())
    candidate3 = QString ();
  
  MultiLabel4->setText ("");
  LineEdit4->setEnabled (FALSE);
  LineEdit4->setText ("");
  MultiCandidate4->setEnabled (FALSE);
  if (!candidate4.isNull ())
    candidate4 = QString ();
  
  MultiLabel5->setText ("");
  LineEdit5->setEnabled (FALSE);
  LineEdit5->setText ("");
  MultiCandidate5->setEnabled (FALSE);
  if (!candidate5.isNull ())
    candidate5 = QString ();

  if (be && ms && ms_list.count () > 0)
    {
      be->findMultiSearchEntry (ms->getMultiId ());
      multi_last_book_ident = be->getIdentity ();
      multi_last_id = ms->getMultiId ();
      // qDebug ("%s:%d", (const char *) , ms->getMultiId ());
    }
}

void
Zten::multiSearch ()
{
  // FIXME: qbookentry interface should have EUC API.
  QTextCodec *codec = QTextCodec::codecForName("eucJP");
  if (current_book)
    {
      QBookMultiSearch *ms = ms_list.at (MultiSearch->currentItem ());
      QList<QString> multi_words;
      multi_words.setAutoDelete (TRUE);
      if (multi_number >= 1)
	multi_words.append(new QString (candidate1.isNull ()?
					codec->fromUnicode (LineEdit1->text ()) :
					candidate1));
      if (multi_number >= 2)
	multi_words.append(new QString (candidate2.isNull ()?
					codec->fromUnicode (LineEdit2->text ()) :
					candidate2));
      if (multi_number >= 3)
	multi_words.append(new QString (candidate3.isNull ()?
					codec->fromUnicode (LineEdit3->text ()) :
					candidate3));
      if (multi_number >= 4)
	multi_words.append(new QString (candidate4.isNull ()?
					codec->fromUnicode (LineEdit4->text ()) :
					candidate4));	
      if (multi_number == 5)
	multi_words.append(new QString (candidate5.isNull ()?
					codec->fromUnicode (LineEdit5->text ()) :
					candidate5));		
      int matches = current_book->multiSearch (ms->getMultiId (), multi_words);
      EntryList2->clear ();
      for (int i = 0; i < matches; i ++)
	{
	  QString heading = current_book->getHeading (i);
	  if (!heading.isNull())
	    {
	      EntryList2->insertItem (new QBookHitItem
				      (heading + " [" +
				       current_book->getTitle () + "]",
				       current_book, i), -1);	  
	    }
	}
      
      MultiTab->setCurrentPage (1);
      toggleContentForce2 (FALSE);

      if (matches == 0)
	{
	  EntryList2->insertItem (new QBookHitItem ("No matches", NULL, -1));
	  EntryList2->setSelectionMode (QListBox::NoSelection);
	}
      else
	{
	  EntryList2->setSelectionMode(QListBox::Single);
	  EntryList2->setSelected(0, true);
	  EntryList2->setFocus();
	  toggleContentForce2 (TRUE);
	}
    }
}

void
Zten::addMultiSearchEntry (QBookEntry *be, int index, QString label,
			   bool have_candidate)
{
  current_book = be;
  multi_number++;
  // qDebug ("%d:%d", index, have_candidate);
  switch (index)
    {
    case 0:
      MultiLabel1->setText (label);
      LineEdit1->setEnabled (TRUE);
      if (have_candidate)
	MultiCandidate1->setEnabled (TRUE);
      break;
    case 1:
      MultiLabel2->setText (label);
      LineEdit2->setEnabled (TRUE);
      if (have_candidate)
	MultiCandidate2->setEnabled (TRUE);
      break;
    case 2:
      MultiLabel3->setText (label);
      LineEdit3->setEnabled (TRUE);
      if (have_candidate)
	MultiCandidate3->setEnabled (TRUE);
      break;
    case 3:
      MultiLabel4->setText (label);
      LineEdit4->setEnabled (TRUE);
      if (have_candidate)
	MultiCandidate4->setEnabled (TRUE);
      break;
    case 4:
      MultiLabel5->setText (label);
      LineEdit5->setEnabled (TRUE);
      if (have_candidate)
	MultiCandidate5->setEnabled (TRUE);
      break;
    default:
      break;
    }
}

void
Zten::addMultiSearch (QBookEntry *be, int multi_id, QString title, int)
{
  //char index[10];
  //sprintf (index, " (%d)", serial_no);
  MultiSearch->insertItem (be->getTitle () + " " + title);
  ms_list.append (new QBookMultiSearch (be, multi_id));	
}

void
Zten::findMultiSearches ()
{
  QBookEntry *be;

  MultiSearch->clear ();
  ms_list.clear ();
  for (be = books.first (); be != NULL; be = books.next ())
    {
      be->findMultiSearch ();
    }
}

void
Zten::findMenues ()
{
  QBookEntry *be;

  MenuSearch->clear ();
  menu_books.clear ();
  for (be = books.first (); be != NULL; be = books.next ())
    {
      if (eb_have_menu (be->getBook ()))
	{
	  MenuSearch->insertItem (be->getTitle ());
	  menu_books.append (be);
	}
    }
}

void
Zten::menuSelected (int index)
{
  if (menu_books.count () > 0)
    {
      MenuSearch->setEnabled (TRUE);
      QBookEntry *be = menu_books.at (index);
      current_book = be;
      menu_last_book_ident = be->getIdentity ();
      gotoMenu (be);
    }
  else
    {
      MenuSearch->setEnabled (FALSE);
    }
}

void
Zten::openBooks ()
{
  if (!opened)
    {
      QListViewItemIterator it (DictionaryList);
      QBookCheckItem *bi;

      for (; it.current();++it)
	{
	  bi = (QBookCheckItem *) it.current ();
	  if (bi->isOn ())
	    {
	      // 	  fprintf (stderr,
	      //   "OPEN %s%s, %d\n",
	      //   bi->getPath (),
	      //   (const char *)bi->text (2),
	      //   bi->getSubBookPos ());
	      QBookEntry *be = new QBookEntry (QString (bi->getPath ()),
					       (const char *)bi->text(2),
					       bi->getSubBookPos(),
					       bi->text(3),
					       bi->text(4),
					       colorGroup ());
	      if (be->isOpened ())
		{
		  // bi->setEnabled (TRUE);
		  be->setFontSize (cur_font.pixelSize ());
		  connect (be, SIGNAL (foundMultiSearch(QBookEntry*,int,QString,int)),
			   this, SLOT (addMultiSearch (QBookEntry*,int,QString,int)));
		  connect (be, SIGNAL (foundMultiSearchEntry(QBookEntry*,int,
							     QString,bool)),
			   this, SLOT (addMultiSearchEntry (QBookEntry*,int,
							    QString,bool)));
		  if (be) books.append(be);
		}
	      else
		{
		  delete be;
		}
	    }
	  opened = TRUE;
	}
    }
}

void
Zten::closeBooks ()
{
  books.clear();
  opened = FALSE;
}

void
Zten::multiResume ()
{
  if (multi_last_book_ident.isNull ())
    {
      MultiSearch->setCurrentItem (0);
      multiSelected (0);
    }
  else
    {
      QBookMultiSearch *ms;
      int count = 0;
      for (ms = ms_list.first (); ms != NULL; ms = ms_list.next ())
	{
	  if (ms->getBookEntry ()->equals (multi_last_book_ident) &&
	      ms->getMultiId () == multi_last_id)
	    {
	      MultiSearch->setCurrentItem (count);
	      multiSelected (count);
	      return;
	    }
	  count++;
	}
      multiSelected (0);
    }
}

void
Zten::menuResume ()
{
  if (menu_last_book_ident.isNull ())
    {
      MenuSearch->setCurrentItem (0);
      menuSelected (0);
    }
  else
    {
      QBookEntry *be;
      int count = 0;
      for (be = menu_books.first (); be != NULL; be = menu_books.next ())
	{
	  if (be->equals (menu_last_book_ident))
	    {
	      MenuSearch->setCurrentItem (count);
	      menuSelected (count);
	      return;
	    }
	  count++;
	}
      menuSelected (0);
    }  
}

void
Zten::tabChanged (QWidget *)
{
  if (MainTab->currentPageIndex () == 0)
    {
      closeBooks ();
      emit booklistSelected ();
    }
  else if (MainTab->currentPageIndex () == 1)
    {
      history.resize (0);
      history_current = -1;
      emit noHistory ();
      current_browser = ContentBrowser;
      ContentBrowser->setText ("");
      current_book = NULL;

      openBooks ();
      emit contentSelected ();

      LineEdit->setFocus();
      searchIncremental (LineEdit->currentText());
    }
  else if (MainTab->currentPageIndex () == 2)
    {
      history.resize (0);
      history_current = -1;
      emit noHistory ();
      current_browser = ContentBrowser2;
      ContentBrowser2->setText ("");
      current_book = NULL;
      EntryList2->clear ();
      toggleContentForce2 (FALSE);
      openBooks ();
      emit contentSelected ();
      MultiTab->setCurrentPage (0);
      MultiLookup->setEnabled (FALSE);
      findMultiSearches ();
      multiResume ();

      LineEdit1->setFocus();
      // multiSelected (0);
    }
  else if (MainTab->currentPageIndex () == 3)
    {
      history.resize (0);
      history_current = -1;
      emit noHistory ();
      current_browser = ContentBrowser3;
      ContentBrowser3->setText ("");
      current_book = NULL;

      openBooks ();
      emit contentSelected ();
      findMenues ();
      menuResume ();
      ContentBrowser3->setFocus();
      // menuSelected (0);
    }
    
}

void
Zten::paste()
{
  if (MainTab->currentPageIndex () != 1)
    {
      MainTab->setCurrentPage (1);
    }
  LineEdit->setEditText (QApplication::clipboard()->text());
}

void
Zten::copy()
{
  current_browser->copy();
}

// void
// Zten::searchClipboard ()
// {
//   LineEdit->setEditText (QApplication::clipboard()->text());
//   search ();
// }
