#ifndef ZTEN_H
#define ZTEN_H

#include <eb/eb.h>
#include <eb/error.h>

#include <stdio.h>
#include <stdlib.h>
#include <qlist.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qfont.h>
#include "ztenbase.h"
#include "qbookentry.h"
#include "qbookitem.h"
#include "qlistboxrichtext.h"
#ifndef QTAPP
#include <qpe/config.h>
#else
#include <qsettings.h>
#endif

#define VERSION "1.6.2"

class QBookHitItem : public QListBoxRichText
{
 public:
  QBookHitItem (const QString &label, QBookEntry *be, int hit_pos);
  ~QBookHitItem ();
  QBookEntry *getBookEntry () { return be; }
  int getHitPos () { return hit_pos; }
 private:
  QBookEntry *be;
  int hit_pos;
};

class QBookMultiSearch
{
 public:
  QBookMultiSearch (QBookEntry *be,int multi_id)
  { this->be = be; this->multi_id = multi_id; }
  ~QBookMultiSearch () {};
  QBookEntry *getBookEntry () { return be; }
  int getMultiId () { return multi_id; }
 private:
  QBookEntry *be;
  int multi_id;
};

class Zten : public ZtenBase
{
  Q_OBJECT
public:
  Zten (QWidget *parent = 0, const char* name = 0, WFlags fl = 0);
  ~Zten ();
  void setCurrentFont (QFont font);
  void setImageScale (int scale);
  void addBook (QBookItem *item);
  void clearBooks ();
  void readConfig ();
  void writeConfig ();
  void lookup (QString key);
private:

#ifdef QTAPP
  QSettings *settings;
#else  
  Config *config;
#endif  
  QList<QBookEntry> books;
  QList<QBookMultiSearch> ms_list; // current multi search list.
  QList<QBookEntry> menu_books;

  bool displaying;
  bool incremental_mode;
  
  // FIXME: QBookBrowser should have these member.
  QBookEntry *current_book;
  QArray<EB_Position> history;
  int history_current;

  void goToNoHistory (EB_Position pos);
  QFont cur_font;
  int cur_scale;
  bool redisplay (QListBoxItem *item = NULL);
  int onItemPos (QBookCheckItem *item);
  bool opened;
  void linkJump (int pos);
  void findMultiSearches ();
  int multi_number;

  QBookBrowser *current_browser;
  
  // FIXME: ...
  bool displaying2;

  QString candidate1;
  QString candidate2;
  QString candidate3;
  QString candidate4;
  QString candidate5;

  QString multi_last_book_ident;
  int multi_last_id;

  QString menu_last_book_ident;
  
public slots:
  void selectBooks ();
  void listUp ();
  void listDown ();
  void setExtension ();
  void deleteBook ();
  void copy ();
  void paste ();
  void back ();
  void forward ();
  void down ();
  void entryListEscape ();
  void entryListUp ();
  void entryListDown ();
  void contentLineUp ();
  void contentLineDown ();
  void contentPageUp ();
  void contentPageDown ();
  void toggleContent ();
  void toggleDirection ();
  // FIXME: ...
  void toggleContent2 ();
  void toggleDirection2 ();
  void entryListUp2 ();
  void entryListDown2 ();
  void contentLineUp2 ();
  void contentLineDown2 ();
  void contentPageUp2 ();
  void contentPageDown2 ();
  void contentLineUp3 ();
  void contentLineDown3 ();
  void contentPageUp3 ();
  void contentPageDown3 ();
  void setIncremental (bool);
  void reload ();
  
  //  void searchClipboard ();

 protected:
  void resizeEvent (QResizeEvent* e);
  bool eventFilter (QObject *o, QEvent *e);
private slots:
  void clearLine ();
  void toggleContentForce (bool);
  void openBooks ();
  void closeBooks ();
  void tabChanged (QWidget*);
  void search ();
  void activateAndSearch ();
  void searchIncremental (const QString &word);
  void goTo (EB_Position);
  void itemSelected (QListBoxItem *item = NULL);
  void itemPressed (QListBoxItem *item = NULL);
  void playSound (int);
  void addMultiSearch (QBookEntry*,int,QString,int);
  void multiSelected (int);
  void addMultiSearchEntry (QBookEntry *,int,QString,bool);
  void multiSearch ();
  void multiResume ();
  void menuResume ();
  // FIXME: ...
  void itemSelected2 (QListBoxItem *item = NULL);
  void itemPressed2 (QListBoxItem *item = NULL);
  void toggleContentForce2 (bool);
  
  void gotoMenu (QBookEntry *);
  void findMenues ();
  void menuSelected (int);

  void multiCandidate1 ();
  void multiCandidate2 ();
  void multiCandidate3 ();
  void multiCandidate4 ();
  void multiCandidate5 ();
  void setCandidateLabel (int, QString, QString);
  
//   int changeDirectory(const char*);
signals:
  // history signal
  void noHistory (); // disable back, forward
  void beginningOfHistory (); // disable back
  void endOfHistory (); // disable forward
  void makeHistory (); // enable back, disable forward
  void followHistory (); // enable back, forward
  // tab signal
  void contentSelected ();
  void booklistSelected ();
  void incrementalIsSet (bool);
};

#endif // ZTEN_H
