#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qlistview.h>
#include <qcheckbox.h>
#include <qlayout.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qapplication.h>
#include "zten.h"
#include "ztenbook.h"
#include "qbookitem.h"
#include "zpixmap.h"

#include "rightarrow.xpm"
#include "leftarrow.xpm"

ZtenBook::ZtenBook (QWidget* parent,
		    const char* name, bool modal, WFlags fl) : 
  ZtenBookBase(parent, name, modal, fl)
{
  codec = QTextCodec::codecForName("eucJP");

  ZtenBookBaseLayout->setStretchFactor(CandidateListView, 1);
  ZtenBookBaseLayout->setStretchFactor(SelectedListView, 1);

  // XXX Hack for stupid layout manager.
  QWidget *d = QApplication::desktop();
  SelectedListView->setMinimumSize(QSize((int) d->width() * 45 / 100,
					 (int) d->height () * 4 / 5)); 

  AddButton->setPixmap (ZPIXMAP(rightarrow_xpm));
  DeleteButton->setPixmap (ZPIXMAP(leftarrow_xpm));

  connect ((QObject *)AddButton, SIGNAL (clicked()), this, SLOT (add ()));
  connect ((QObject *)DeleteButton, SIGNAL (clicked()), this, SLOT (del ()));
#ifdef QTAPP
  QHBoxLayout *bl = new QHBoxLayout;
  bl->setSpacing (6);
  bl->setMargin (0);
  bl->addItem (new QSpacerItem(20, 20,
			       QSizePolicy::Expanding,
			       QSizePolicy::Minimum));

  QPushButton *okButton = new QPushButton( this, "OkButton" );
  okButton->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)1,
				      (QSizePolicy::SizeType)1,
				      AddButton->sizePolicy().hasHeightForWidth()));
  okButton->setText( tr( "OK" ) );
  bl->addWidget( okButton );

  QPushButton *cancelButton = new QPushButton( this, "CancelButton" );
  cancelButton->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)1,
					  (QSizePolicy::SizeType)1,
					  DeleteButton->sizePolicy().hasHeightForWidth()));
  cancelButton->setText( tr( "Cancel" ) );
  bl->addWidget( cancelButton );

  bl->addItem (new QSpacerItem(20, 20,
			       QSizePolicy::Expanding,
			       QSizePolicy::Minimum));
  
  ZtenBookBaseLayout->addLayout( bl );
  connect ((QObject *)okButton, SIGNAL (clicked()),
	   this, SLOT (accept ()));
#endif
  
  collectBooks ();
  
}

ZtenBook::~ZtenBook ()
{
}

// Botsu.
// void
// ZtenBook::resizeEvent (QResizeEvent* e)
// {
//   QSize s = e->size ();
//   if (s.width () > s.height ())
//     {
//       CenterLayout->setDirection (QBoxLayout::LeftToRight);      
//       ButtonLayout->setDirection (QBoxLayout::LeftToRight);      
//       ZtenBookBaseLayout->setDirection (QBoxLayout::LeftToRight);
//     }
//   else
//     {
//       CenterLayout->setDirection (QBoxLayout::LeftToRight);      
//       ButtonLayout->setDirection (QBoxLayout::LeftToRight);            
//       ZtenBookBaseLayout->setDirection (QBoxLayout::TopToBottom);      
//     }
//   ZtenBookBase::resizeEvent (e);
// }

void
ZtenBook::add ()
{
  QBookItem *item = (QBookItem *)CandidateListView->selectedItem ();
  if (item)
    CandidateListView->takeItem ((QListViewItem *)item);
  if (item)
    SelectedListView->insertItem ((QListViewItem *)item);
  if (CandidateListView->currentItem ())
    CandidateListView->setSelected (CandidateListView->currentItem (), TRUE);
}

void
ZtenBook::del ()
{
  QBookItem *item = (QBookItem *)SelectedListView->selectedItem ();
  if (item)
    SelectedListView->takeItem ((QListViewItem *)item);
  if (item)
    CandidateListView->insertItem ((QListViewItem *)item);
}

void
ZtenBook::addSelectedBook (QBookCheckItem *item)
{
  QBookItem *newItem = new QBookItem (SelectedListView);

  newItem->setText (0, item->text (1));
  newItem->setText (1, item->text (2));
  newItem->setSubBookPos (item->getSubBookPos ());
  newItem->setPath (item->getPath ());
  newItem->setText (2, item->text (5));

  newItem->setChecked (item->isOn ());
  newItem->setExtension (item->text (3));
  newItem->setAppendix (item->text (4));

  // Move to the bottom;
  QListViewItem *last;
  for (last = SelectedListView->firstChild ();
       last && last->itemBelow ();
       last = last->itemBelow ());
  newItem->moveItem (last);

  QBookItem *delItem = searchBook (item->getPath (),
				   (const char *)item->text (2),
				   item->getSubBookPos ());
  CandidateListView->removeItem (delItem);
  
}

void
ZtenBook::collectBooks ()
{
  const char *curd;
  int i = 0;
  for (curd = g_book_path[i].path; curd != NULL; curd = g_book_path[++i].path)
    {
      struct dirent **dir;
      int k, j, n_files;

      EB_Book book;
      EB_Subbook_Code subbook_list[EB_MAX_SUBBOOKS];
      int subbook_count;
      char title[EB_MAX_TITLE_LENGTH + 1];

      if (! (chdir(curd) < 0))
	{
	  if ((n_files = scandir(".", &dir, 0, alphasort)) < 0)
	    free(dir);
	  else
	    for(k = 0; k < n_files; k++)
	      {
		if (strcmp(dir[k]->d_name, ".") == 0 || 
		    strcmp(dir[k]->d_name, "..") == 0) 
		  continue;
		eb_initialize_book (&book);
		if (eb_bind (&book, dir[k]->d_name) != EB_SUCCESS)
		  {
		    eb_finalize_book (&book);
		    free (dir[k]);
		    continue;
		  }
		if (eb_subbook_list (&book, subbook_list, &subbook_count)
		    == EB_SUCCESS)
		  {
		    for (j = 0; j < subbook_count; j++)
		      {
			if (eb_subbook_title2 (&book, subbook_list[j],
					       title) == EB_SUCCESS)
			  {
			    QBookItem* item;
			    
			    item = new QBookItem (CandidateListView);
			    item->setText (0, codec->toUnicode(title));
			    item->setText (1, dir[k]->d_name);
			    // item->setText (2, "");
			    item->setSubBookPos(j);
			    item->setPath(curd);
			    item->setText (2, g_book_path[i].name);
			    CandidateListView->insertItem (item);
			  }
		      }
		  }
		eb_finalize_book(&book);
		free(dir[k]);
	      }
	}
    }
}

QBookItem *
ZtenBook::searchBook (const char *path, const char *book, int sub_pos)
{
  QListViewItemIterator it (CandidateListView);
  //  fprintf (stderr, "path:%s, book:%s, pos:%d\n", path, book, sub_pos);
  for (; it.current(); ++it)
    {
      QBookItem *bi = (QBookItem *) it.current ();
      if ((strcmp (path, bi->getPath ()) == 0) &&
	  (strcmp (book, (const char *) bi->text (1)) == 0) &&
	  bi->getSubBookPos () == sub_pos)
	return bi;
    }
  return NULL;
}

void
ZtenBook::accept ()
{
  QListViewItemIterator it (SelectedListView);
  QBookItem *bi;
  Zten *zten = (Zten *)parent ();

  zten->clearBooks ();
  for (; it.current(); ++it)
    {
      bi = (QBookItem *) it.current ();
      zten->addBook (bi);
    }
  ZtenBookBase::accept();
}

