#include "ztencandidate.h"
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qimage.h>
#include <eb/error.h>
#include <eb/text.h>
#include <eb/font.h>
#include <stdio.h>
#include "qlistboxrichtext.h"

#include "zpixmap.h"
#include "back.xpm"
#include "forward.xpm"

#define SEPARATOR     "\a"

EB_Error_Code
candidate_hook_end (EB_Book* book, EB_Appendix*, void *obj, 
		    EB_Hook_Code code,
		    int, const unsigned int* argv)
{
  ZtenCandidate *candidate = (ZtenCandidate *) obj;
  EB_Position pos;

  if (code == EB_HOOK_END_CANDIDATE_LEAF)
    {
      pos.page = -1;
      pos.offset = -1;
    }
  else if (code == EB_HOOK_END_CANDIDATE_GROUP)
    {
      pos.page = argv[1];
      pos.offset = argv[2];
    }
  eb_write_text_string (book, SEPARATOR);
  candidate->addCandidate (eb_current_candidate (book), pos);
  return 0;
}

EB_Error_Code
candidate_hook_null (EB_Book*, EB_Appendix*, void *, EB_Hook_Code, 
		     int, const unsigned int*)
{
  return 0;
}

EB_Error_Code
candidate_hook_narrow_jisx0208 (EB_Book* book, EB_Appendix *appendix, 
				void *container, 
				EB_Hook_Code code, 
				int argc, const unsigned int* argv)
{
  if (argv[0] == 0xa1e3)
    eb_write_text_string(book, "&lt;");
  else if (argv[0] == 0xa1e4)
    eb_write_text_string(book, "&gt;");
  else 
    eb_hook_euc_to_ascii(book, appendix, container, code, argc, argv);
  return 0;
}

EB_Error_Code
candidate_hook_wide_font (EB_Book* book, EB_Appendix*, void *obj, 
			  EB_Hook_Code,
			  int, const unsigned* argv)
{
  char c [200];
  char bitmap[EB_SIZE_WIDE_FONT_48];
#if 0
  char image_data[EB_SIZE_FONT_IMAGE];
  size_t image_length;
#endif
  ZtenCandidate *candidate = (ZtenCandidate *) obj;

  EB_Error_Code rc = eb_wide_font_character_bitmap(book, argv[0], bitmap);
  if (rc != EB_SUCCESS)
    {
      eb_write_text_string (book, "  ");
      return 0;
    }
  // FIXME: there should be a better way -- from qtjiten
  QImage qim(16, 16, 1, 2, QImage::BigEndian);
  qim.setColor(0, 0xffffff);
  qim.setColor(1, 0x000000);
  for(int y=0; y<16; y++) 
    for(int x=0; x<16; x++)
      qim.setPixel(x, y, (bitmap[y*2+((x<8)?0:1)] & (1 << (7-(x%8)))) ? 1:0);
  sprintf(c, "n-font_w%x", argv[0]);
  QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));

  // Highlighted
  qim = QImage (16, 16, 32, 2, QImage::BigEndian);
  QColorGroup g = candidate->colorGroup ();
  int back = g.highlight ().rgb ();
  int fore = g.highlightedText ().rgb ();
  for(int y=0; y<16; y++) 
    for(int x=0; x<16; x++)
      qim.setPixel(x, y, (bitmap[y*2+((x<8)?0:1)] & (1 << (7-(x%8)))) ? fore:back);
  sprintf(c, "h-font_w%x", argv[0]);
  QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));
  
  sprintf(c, "<img src=\"n-font_w%x\" alt=\"GAIJI(%x)\" width=%d height=%d>",
	  argv[0], argv[0],
	  candidate->font().pixelSize (),
	  candidate->font().pixelSize ());
  eb_write_text_string(book, c);
  return 0;
}

EB_Error_Code
candidate_hook_narrow_font (EB_Book* book, EB_Appendix*, void *obj, 
			    EB_Hook_Code,
			    int, const unsigned* argv)
{
  char c [200];
  char bitmap[EB_SIZE_NARROW_FONT_48];
  ZtenCandidate *candidate = (ZtenCandidate *) obj;

  EB_Error_Code rc = eb_narrow_font_character_bitmap(book, argv[0], bitmap);
  if (rc != EB_SUCCESS)
    {
      eb_write_text_string (book, " ");
      return 0;
    }
  // FIXME: there should be a better way -- from qtjiten
  QImage qim(8, 16, 1, 2, QImage::BigEndian);
  qim.setColor(0, 0xffffff);
  qim.setColor(1, 0x000000);
  for(int y=0; y<16; y++) 
    for(int x=0; x<8; x++)
      qim.setPixel(x, y, (bitmap[y] & (1 << (7-x))) ? 1:0);
  sprintf(c, "n-font_n%x", argv[0]);
  QMimeSourceFactory::defaultFactory()->setImage(c, qim.convertDepth(32));

  // highlighted;
  qim = QImage (8, 16, 32, 2, QImage::BigEndian);
  QColorGroup g = candidate->colorGroup ();
  int back = g.highlight ().rgb ();
  int fore = g.highlightedText ().rgb ();
  for(int y=0; y<16; y++) 
    for(int x=0; x<8; x++)
      qim.setPixel(x, y, (bitmap[y] & (1 << (7-x))) ? fore:back);
  sprintf(c, "h-font_n%x", argv[0]);
  QMimeSourceFactory::defaultFactory()->setImage(c, qim);
  
  sprintf(c, "<img src=\"n-font_n%x\" alt=\"GAIJI(%x)\" width=%d height=%d>",
	  argv[0], argv[0],
	  candidate->font().pixelSize () / 2,
	  candidate->font().pixelSize ());
  eb_write_text_string(book, c);
  
  return 0;
}

void
ZtenCandidate::addCandidate (QString text, EB_Position pos)
{
  candidate_list.append (new QString (text));
  pos_list.resize(pos_list.size() + 1);
  pos_list [pos_list.size () - 1] = pos;
}

ZtenCandidate::ZtenCandidate (EB_Book *book,
			      EB_Position pos,
			      int index,
			      QWidget* parent,
			      const char* name, bool modal, WFlags fl) :
  ZtenCandidateBase(parent, name, modal, fl)			      
{
  codec = QTextCodec::codecForName ("eucJP");
  EB_Hook candidate_hooks [] =
    {
      {EB_HOOK_NEWLINE, candidate_hook_null},
      {EB_HOOK_NARROW_JISX0208, candidate_hook_narrow_jisx0208},
      {EB_HOOK_NARROW_FONT, candidate_hook_narrow_font},
      {EB_HOOK_WIDE_FONT, candidate_hook_wide_font},
      {EB_HOOK_END_CANDIDATE_LEAF, candidate_hook_end},
      {EB_HOOK_END_CANDIDATE_GROUP, candidate_hook_end},
      {EB_HOOK_NULL, NULL}
    };
  eb_initialize_hookset (&candidate_hookset);

  BackButton->setPixmap (ZPIXMAP (back_xpm));
  ForwardButton->setPixmap (ZPIXMAP (forward_xpm));
  eb_set_hooks (&candidate_hookset, candidate_hooks);

  this->book = book;
  this->index = index;

  history.resize (0);
  history_current = -1;
  candidate_list.setAutoDelete (TRUE);  
  setContent (pos);
  
  connect ((QObject *) BackButton, SIGNAL (clicked ()), this, SLOT (back ()));
  connect ((QObject *) ForwardButton, SIGNAL (clicked ()), this, SLOT (forward ()));
  connect ((QObject *) Candidates, SIGNAL (clicked (QListBoxItem*)),
	   this, SLOT (jump(QListBoxItem*)));
  connect ((QObject *) Candidates, SIGNAL (returnPressed (QListBoxItem*)),
	   this, SLOT (jump(QListBoxItem*)));
  
  BackButton->setEnabled (FALSE);
  ForwardButton->setEnabled (FALSE);
}

ZtenCandidate::~ZtenCandidate ()
{
  eb_finalize_hookset (&candidate_hookset);
}

void
ZtenCandidate::setContent (EB_Position pos)
{
  if (history_current == (int) history.size() - 1)
    {
      history.resize(history.size() + 1);
    }
  history [++history_current] = pos;
  if (history.size () != 1)
    {
      BackButton->setEnabled (TRUE);
    }
  setContentNoHistory (pos);
}

void
ZtenCandidate::setContentNoHistory (EB_Position pos)
{
  QCString s, c;
  char text [MAX_CANDIDATE+1];
  int text_length;
  EB_Position p0;
  pos_list.resize (0);
  candidate_list.clear ();
  Candidates->clear ();

  if (eb_tell_text (book, &p0) != EB_SUCCESS)
    p0.page = -1;
  if (eb_seek_text (book, &pos) != EB_SUCCESS)
    return;
  int bytes = 0;
  while (1)
    {
      if (eb_read_text(book, NULL, &candidate_hookset, this,
		       MAX_CANDIDATE, text, &text_length) != EB_SUCCESS)
	{
	  bytes = 0;
	  break;
	}
      bytes += text_length;
      if (text_length < 1)
	break;
      s.append(text);
    }
  if (bytes == 0)
    {
      if (p0.page >= 0)
	eb_seek_text(book, &p0);
      return;
    }
  QString str = codec->toUnicode (s);
  QStringList strlist = QStringList::split (SEPARATOR, str);
  for (unsigned int i = 0; i < strlist.count (); i++)
    {
      // XXX It causes mojibake.
      QString text = codec->toUnicode (*candidate_list.at (i));
      Candidates->insertItem (new QListBoxRichText (text,
						    strlist[i]));
    }
  if (strlist.count () > 0)
    {
      Candidates->setSelected(0, true);
      Candidates->setFocus();
    }
}

void
ZtenCandidate::jump (QListBoxItem *)
{
  int page;
  int offset;

  EB_Position pos = pos_list[Candidates->currentItem ()];

  page = pos.page;
  offset = pos.offset;

  if (page >= 0)
    {
      pos.page = page;
      pos.offset = offset;
      setContent (pos);
    }
  else
    {
      if (!Candidates->currentText ().isNull ())
	{
	  emit candidateDecided (index,
				 *candidate_list.at (Candidates->currentItem ()),
				 Candidates->currentText ());
	}
      ZtenCandidateBase::accept ();
    }
}

void
ZtenCandidate::back ()
{
  if (history_current >= 1)
    {
      history_current--;
      EB_Position p = history[history_current];
      setContentNoHistory (p);
    }
  if (history_current == 0)
    {
      BackButton->setEnabled (FALSE);
    }
  ForwardButton->setEnabled (TRUE);
}

void
ZtenCandidate::forward ()
{
  if (history_current < (int) history.size() - 1)
    {
      history_current++;
      EB_Position p = history[history_current];
      setContentNoHistory (p);
    }
  if (history_current == (int) history.size() - 1)
    {
      emit ForwardButton->setEnabled (FALSE);
    }
}

void
ZtenCandidate::accept ()
{
  jump (NULL);
}
