#include <dirent.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <qlistview.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include "zten.h"
#include "ztenext.h"
#include <eb/appendix.h>

ZtenExtension::ZtenExtension (const char* filter,
			      const char* appendix_name,
			      QWidget* parent, 
			      const char* name,
			      bool modal,
			      WFlags fl) : 
  ZtenExtensionBase(parent, name, modal, fl)
{
  const char *curd;
  int i = 0;
  int n_files, k, count = 0;
  EB_Appendix appendix;
  int set_pos = -1;

  QCString hiragana ("Hiragana");
  if (filter == hiragana)
    {
      HiraganaCheckBox->setChecked(TRUE);
    }
  connect (AppendixCheckBox, SIGNAL (toggled (bool)), this,
	   SLOT (appendixToggled (bool)));  
  for (curd = g_appendix_path[i].path;
       curd != NULL; curd = g_appendix_path[++i].path)
    {
      struct dirent **dir;
      if (! (chdir(curd) < 0))
	{
	  int pos = 0;
	  if ((n_files = scandir(".", &dir, 0, alphasort)) < 0)
	    free(dir);
	  else
	    for(k = 0; k < n_files; k++)
	      {
		if (strcmp(dir[k]->d_name, ".") == 0 || 
		    strcmp(dir[k]->d_name, "..") == 0) 
		  continue;
		eb_initialize_appendix (&appendix);
		if (eb_bind_appendix (&appendix, dir[k]->d_name) != EB_SUCCESS)
		  {
		    eb_finalize_appendix (&appendix);
		    free(dir[k]);
		    continue;
		  }
		QString set_appendix (dir[k]->d_name);
		if (g_appendix_path[i].name == NULL)
		  // 'directory_name/appendix_name'
		  {
		    set_appendix = curd + set_appendix;
		  }
		else
		  {
		    set_appendix = set_appendix + "(" +
		      g_appendix_path[i].name + ")";
		  }
		if (set_appendix == appendix_name) set_pos = pos;
		AppendixComboBox->insertItem (set_appendix);
		pos++;
		count++;
		eb_finalize_appendix (&appendix);
		free(dir[k]);
	      }
	}
      if (set_pos >= 0)
	{
	  AppendixComboBox->setCurrentItem (set_pos);
	}
    }
  if (count == 0)
    {
      AppendixCheckBox->setEnabled (FALSE);
      AppendixComboBox->setEnabled (FALSE);
    }
  else if (!(appendix_name == NULL || strlen (appendix_name) == 0))
    {
      AppendixCheckBox->setChecked(TRUE);
    }
}

ZtenExtension::~ZtenExtension ()
{
}

void ZtenExtension::appendixToggled (bool on)
{
  if (on)
    {
      AppendixComboBox->setEnabled (TRUE);
    }
  else
    {
      AppendixComboBox->setEnabled (FALSE);
    }
}

void ZtenExtension::accept ()
{
  QListViewItem *item = ((Zten *) parentWidget())->DictionaryList->selectedItem ();
  if (HiraganaCheckBox->isChecked ())
    {
      item->setText (3, "Hiragana");
    }
  else
    {
      item->setText (3, "");
    }
  if (AppendixCheckBox->isChecked () && AppendixComboBox->currentText () != "")
    {
      item->setText (4, AppendixComboBox->currentText ());
    }
  else
    {
      item->setText (4, "");
    }
  ZtenExtensionBase::accept();
}
