#include <qiconset.h>
#include <qimage.h>
#include <qpixmap.h>
#ifdef QTAPP
#include <qtoolbar.h>
#include <qmenubar.h>
#define QPEToolBar QToolBar
#define QPEMenuBar QMenuBar
#else
#include <qpe/qpetoolbar.h>
#include <qpe/qpemenubar.h>
#endif
#include <qaction.h>
#include <qmenubar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qfont.h>
#include <qapplication.h>
#include <qclipboard.h>
#ifndef QTAPP
#include <qcopchannel_qws.h>
#endif
#include "ztenwindow.h"
#include "zpixmap.h"

#include "edit.xpm"
#include "1downarrow.xpm"
#include "1uparrow.xpm"
#include "gear.xpm"
#include "edittrash.xpm"
#include "editcopy.xpm"
#include "editpaste.xpm"
#include "back.xpm"
#include "forward.xpm"
#include "down.xpm"
//#include "gohome.xpm"

#define NUM_SCALES 7
int scales [] = {10, 25, 50, 75, 100, 150, 200};

struct font_entry
{
  const char *family;
  int size;
} font_list [] =
  {
    {"lcfont", 10},
    {"lcfont", 12},
    {"lcfont", 16},
    {"lcfont", 18},
    {"lcfont", 20},
    {"lcfont", 24},
    {NULL, 0},
  };

ZtenWindow::ZtenWindow (QWidget *parent) : QMainWindow (parent)
{
  setCaption ("Zten");
  setToolBarsMovable(FALSE);
  QPEToolBar *topBar = new QPEToolBar (this);

  topBar->setHorizontalStretchable(TRUE);

  QPEMenuBar *menuBar = new QPEMenuBar (topBar);
  menuBar->setMargin (0);
  QPEToolBar *toolBar;

  toolBar = new QPEToolBar (this);

  selectBooksAction = new QAction (tr ("Select Books"),
				   ZPIXMAP (edit_xpm),
				   QString::null, 0, this, 0);
  connect (selectBooksAction, SIGNAL (activated ()), this,
	   SLOT (selectBooks ()));

  listDownAction = new QAction (tr ("Move Down"),
				ZPIXMAP (downarrow_xpm),
				QString::null, 0, this, 0);
  connect (listDownAction, SIGNAL (activated ()), this, SLOT (listDown ()));

  listUpAction = new QAction (tr ("Move Up"),
			      ZPIXMAP (uparrow_xpm),
			      QString::null, 0, this, 0);
  connect (listUpAction, SIGNAL (activated ()), this, SLOT (listUp ()));

  extensionAction = new QAction (tr ("Set Extension"),
				ZPIXMAP (gear_xpm),
				QString::null, 0, this, 0);
  connect (extensionAction, SIGNAL (activated ()), this,
	   SLOT (setExtension ()));

  //  homeAction = new QAction (tr ("Goto Menu"),
  //			    ZPIXMAP (gohome_xpm),
  //			    QString::null, 0, this, 0);
  //  connect (homeAction, SIGNAL (activated ()), this, SLOT (gotoMenu ()));
  
  deleteBookAction = new QAction (tr ("Delete Book"),
				  ZPIXMAP (edittrash_xpm),
				  QString::null, 0, this, 0);
  connect (deleteBookAction, SIGNAL (activated ()), this,
	   SLOT (deleteBook ()));
  
  copyAction = new QAction (tr ("Copy"),
			    ZPIXMAP (editcopy_xpm),
			    QString::null, 0, this, 0);
  connect (copyAction, SIGNAL (activated ()), this, SLOT (copy ()));

  pasteAction = new QAction (tr ("Paste"),
			     ZPIXMAP (editpaste_xpm),
			     QString::null, 0, this, 0);
  connect (pasteAction, SIGNAL (activated ()), this, SLOT (paste ()));

  backAction = new QAction (tr ("Back"),
			    ZPIXMAP (back_xpm),
			    QString::null, 0, this, 0);
  connect (backAction, SIGNAL (activated ()), this, SLOT (back ()));

  forwardAction = new QAction (tr ("Forward"),
			       ZPIXMAP (forward_xpm),
			       QString::null, 0, this, 0);
  connect (forwardAction, SIGNAL (activated ()), this, SLOT (forward ()));
  
  downAction = new QAction (tr ("Next"),
			    ZPIXMAP (down_xpm),
			    QString::null, 0, this, 0);
  connect (downAction, SIGNAL (activated ()), this, SLOT (down ()));

  QPopupMenu *file = new QPopupMenu (this);
  if (!ZSMALLDISPLAY)
    menuBar->insertItem (tr("File"), file);

  selectBooksAction->addTo (toolBar);
  listDownAction->addTo (toolBar);
  listUpAction->addTo (toolBar);
  extensionAction->addTo (toolBar);
  deleteBookAction->addTo (toolBar);
  //homeAction->addTo (toolBar);
  copyAction->addTo (toolBar);
  pasteAction->addTo (toolBar);
  backAction->addTo (toolBar);
  forwardAction->addTo (toolBar);
  downAction->addTo (toolBar);
  
  selectBooksAction->addTo (file);
  listDownAction->addTo (file);
  listUpAction->addTo (file);
  extensionAction->addTo (file);
  deleteBookAction->addTo (file);
  //homeAction->addTo (file);
  copyAction->addTo (file);
  pasteAction->addTo (file);
  backAction->addTo (file);
  forwardAction->addTo (file);
  downAction->addTo (file);

  setting = new QPopupMenu (this);
  setting->setCheckable (true);

  menuBar->insertItem (tr("Setting"), setting);
  incremental_id = setting->insertItem (tr("Incremental Search"), this,
					SLOT(toggleIncremental()));
  
  font = new QPopupMenu(this);
  setting->insertItem (tr("Font"), font);

  char s[16];
  int i = 0;
  const char *family;
  for (family = font_list[i].family; family != NULL;
       family = font_list[++i].family)
    {
      int size = font_list[i].size;
      sprintf (s, "%s %d", family, size);
      font->insertItem (s, this, SLOT (setCurrentFont (int)));
    }

  scale = new QPopupMenu(this);
  setting->insertItem (tr("Image scale"), scale);
  
  for (i = 0; i < NUM_SCALES; i++)
    {
      sprintf (s, "%d %%", scales[i]);
      scale->insertItem (s, this, SLOT (setImageScale (int)));
    }
  
  zten = new Zten (this);
  setCentralWidget (zten);
  
  connect (zten, SIGNAL (noHistory ()), this, SLOT (disableNavigations()));
  connect (zten, SIGNAL (beginningOfHistory ()), this, SLOT (disableBack ()));
  connect (zten, SIGNAL (endOfHistory ()), this, SLOT (disableForward()));
  connect (zten, SIGNAL (makeHistory ()), this, SLOT (enableBackNavigations()));
  connect (zten, SIGNAL (followHistory ()), this, SLOT (enableNavigations()));
  connect (zten, SIGNAL (contentSelected ()), this, SLOT (enableContentOperations()));
  connect (zten, SIGNAL (booklistSelected ()), this, SLOT (enableListOperations()));
  connect (zten, SIGNAL (incrementalIsSet (bool)), this, SLOT (setIncremental(bool)));

#ifndef QTAPP  
  // Receive QCOP message.
  QCopChannel *channel = new QCopChannel( "QPE/Application/zten", this );
  connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
	  this, SLOT(dispatchMessage(const QCString&, const QByteArray&)));  
#endif
  zten->readConfig ();
}

void
ZtenWindow::dispatchMessage (const QCString &msg, const QByteArray &data)
{
  if (msg == "lookup(QString)")
    {
      QDataStream stream (data, IO_ReadOnly);
      QString key;
      stream >> key;
      lookup (QString (key));
    }
  else if (msg == "lookup()")
    {
      lookup ();
    }
}

void
ZtenWindow::lookup (QString key)
{
  showMaximized ();
  raise ();
  zten->lookup (key);
}

void
ZtenWindow::lookup ()
{
  showMaximized ();
  raise ();
  zten->lookup (QApplication::clipboard()->text());
}

void
ZtenWindow::resume ()
{
  zten->reload ();
}

void ZtenWindow::enableNavigations ()
{
  // enable back, forward
  backAction->setEnabled (TRUE);
  forwardAction->setEnabled (TRUE);
}
void
ZtenWindow::enableBackNavigations ()
{
  // enable back, disable forward
  backAction->setEnabled (TRUE);
  forwardAction->setEnabled (FALSE);
}
void
ZtenWindow::disableNavigations ()
{
  // disable back, forward
  backAction->setEnabled (FALSE);
  forwardAction->setEnabled (FALSE);
}
void
ZtenWindow::disableBack ()
{
  // disable back
  backAction->setEnabled (FALSE);
}
void
ZtenWindow::disableForward ()
{
  // disable forward
  forwardAction->setEnabled (FALSE);
}

void
ZtenWindow::enableListOperations ()
{
  selectBooksAction->setEnabled (TRUE);
  listUpAction->setEnabled (TRUE);
  listDownAction->setEnabled (TRUE);
  extensionAction->setEnabled (TRUE);
  deleteBookAction->setEnabled (TRUE);
  copyAction->setEnabled (FALSE);
  pasteAction->setEnabled (FALSE);
  backAction->setEnabled (FALSE);
  forwardAction->setEnabled (FALSE);
  downAction->setEnabled (FALSE);
}

void
ZtenWindow::enableContentOperations ()
{
  selectBooksAction->setEnabled (FALSE);
  listUpAction->setEnabled (FALSE);
  listDownAction->setEnabled (FALSE);
  extensionAction->setEnabled (FALSE);
  deleteBookAction->setEnabled (FALSE);
  copyAction->setEnabled (TRUE);
  pasteAction->setEnabled (TRUE);  
  backAction->setEnabled (FALSE);
  forwardAction->setEnabled (FALSE);
  downAction->setEnabled (TRUE);  
}

void
ZtenWindow::setIncremental (bool on)
{
  if (on)
    {
      setting->setItemChecked (incremental_id, true);
    }
  else
    {
      setting->setItemChecked (incremental_id, false);
    }
}

void
ZtenWindow::toggleIncremental ()
{
  if (setting->isItemChecked (incremental_id))
    {
      zten->setIncremental (false);
    }
  else
    {
      zten->setIncremental (true);
    }
}

void
ZtenWindow::setCurrentFont (int i)
{
  i = font->indexOf (i);
  zten->setCurrentFont (QFont(font_list[i].family, font_list[i].size));
}

void
ZtenWindow::setImageScale (int i)
{
  i = scale->indexOf (i);
  zten->setImageScale (scales[i]);
}

void
ZtenWindow::selectBooks ()
{
  zten->selectBooks ();
}

void
ZtenWindow::listDown ()
{
  zten->listDown ();
}

void
ZtenWindow::listUp ()
{
  zten->listUp ();
}

void
ZtenWindow::setExtension ()
{
  zten->setExtension ();
}

void
ZtenWindow::deleteBook ()
{
  zten->deleteBook ();
}

void
ZtenWindow::copy ()
{
  zten->copy();
}

void
ZtenWindow::paste ()
{
  zten->paste();
}

void
ZtenWindow::forward ()
{
  zten->forward();
}

void
ZtenWindow::back ()
{
  zten->back();
}

void
ZtenWindow::down ()
{
  zten->down();
}
